from enum import Enum
from sdgs_tools.dashboard.utils import make_str_to_enum, make_dunder_str

MAPPING = {
    "Semua": "1",
    "Sebagian Besar": "2",
    "Tidak": "3",
}


class Diekspor(Enum):
    SEMUA = "1"
    SEBAGIAN_BESAR = "2"
    TIDAK = "3"

    from_str = classmethod(make_str_to_enum(MAPPING, "3"))
    __str__ = make_dunder_str(MAPPING, "Tidak")



from openpyxl.worksheet.worksheet import Worksheet
from typing import Dict, Optional
from uiautomator2 import Device, UiObject


def get_text(ui: UiObject) -> Optional[str]:
    childText = ui.child(className="android.widget.TextView")
    if childText.exists:
        return childText.info.get("text")
    return ui.info.get("text")


def d_get_text(d: Device, resourceId: str, scroll: bool = True):
    current = d(resourceId=resourceId)
    if scroll and not current.exists:
        d.xpath("//android.widget.ScrollView").scroll()
    if resourceId.startswith("com.kemendes.survey:id/txt"):
        if not current.exists:
            return None
        return current.info.get("text")
    childText = current.child(className="android.widget.TextView")
    if not childText.exists:
        return None
    return childText.info.get("text")


def swipe_box(d: Device, box: UiObject):
    bounds: Dict[str, int] = box.info.get("bounds")
    if not bounds:
        return
    top = bounds["top"]
    right = bounds["right"]
    bottom = bounds["bottom"]
    left = bounds["left"]
    middle = left + ((right - left) // 2)
    d.swipe(middle, bottom, middle, top)


def set_ws_header(ws: Worksheet, header: Dict[str, str], row: int = 1):
    for col, nama in header.items():
        ws[f"{col}{row}"] = nama.title()


def menu_to(d: Device, text: str, resourceId="com.kemendes.survey:id/tabMenu"):
    tabMenu = d(resourceId=resourceId)
    menu = d(text=text)
    if menu.exists:
        return menu.click()
    tabMenu.fling.horiz.backward()
    if menu.exists:
        return menu.click()
    tabMenu.fling.horiz.forward()
    if menu.exists:
        return menu.click()
    raise ValueError(f"Menu tidak ditemukan {text}")



from enum import Enum
from sdgs_tools.dashboard.utils import make_str_to_enum, make_dunder_str

MAPPING = {
    "Petani Pemilik Lahan": "1",
    "Petani Penyewa": "2",
    "Buruh Tani": "3",
    "Nelayan Pemilik Kapal": "4",
    "Nelayan Penyewa Kapal": "5",
    "Buruh Nelayan": "6",
    "Guru": "7",
    "Guru Agama": "8",
    "Pedagang": "9",
    "Pengolahan Industri": "10",
    "Pns": "11",
    "Tni": "12",
    "Perangkat Desa": "13",
    "Pegawai Kantor Desa": "14",
    "Tki": "15",
    "Lainnya": "other",
}


class PekerjaanUtama(Enum):
    PETANI_PEMILIK_LAHAN = "1"
    PETANI_PENYEWA = "2"
    BURUH_TANI = "3"
    NELAYAN_PEMILIK_KAPAL = "4"
    NELAYAN_PENYEWA_KAPAL = "5"
    BURUH_NELAYAN = "6"
    GURU = "7"
    GURU_AGAMA = "8"
    PEDAGANG = "9"
    PENGOLAHAN_INDUSTRI = "10"
    PNS = "11"
    TNI = "12"
    PERANGKAT_DESA = "13"
    PEGAWAI_KANTOR_DESA = "14"
    TKI = "15"
    LAINNYA = "other"

    from_str = classmethod(make_str_to_enum(MAPPING))
    __str__ = make_dunder_str(MAPPING)



import cattr
import click
from datetime import date, datetime
from dateutil import parser
from enum import Enum
from typing import Any, Dict, Type, TypeVar, Union


def str_to_date(value: Union[str, date], t=None) -> date:
    if isinstance(value, date):
        return value
    return parser.parse(
        timestr=value,
        parserinfo=parser.parserinfo(dayfirst=False, yearfirst=True),
    ).date()


def date_to_str(value: date) -> str:
    return value.strftime("%Y-%m-%d")


def parse_datetime(text: str, t) -> datetime:
    return parser.parse(
        timestr=text,
        parserinfo=parser.parserinfo(dayfirst=False, yearfirst=True),
    )


def register_cattr_hooks():
    cattr.register_structure_hook(date, str_to_date)
    cattr.register_structure_hook(datetime, parse_datetime)
    cattr.register_unstructure_hook(date, date_to_str)


E = TypeVar("E", bound=Enum)


def isequal(a: Any, b: str) -> bool:
    if a is None:
        return False
    elif isinstance(a, str) and a.strip().lower() == b.lower():
        return True
    return a == b


def make_str_to_enum(mapping: Dict[str, Any], default_value: Union[str, int] = "1"):
    def str_to_enum(cls: Type[E], val: Union[str, int], t: Type[E] = None) -> E:
        if val is None:
            click.echo(f"Kolom kosong, akan diisi {cls(default_value)}")
            return cls(default_value)
        elif isinstance(val, int):
            try:
                return cls(val)
            except ValueError:
                return cls(str(val))
        for key, enum_val in mapping.items():
            if isequal(val, key):
                return cls(enum_val)
        click.echo(
            f"Isian tidak cocok dengan Referensi : {val}! Akan diisi {cls(default_value)}!"
        )
        return cls(default_value)

    return str_to_enum


def make_dunder_str(mapping: Dict[str, Any], default: Any = None):
    def dunder_str(self: Type[E]):
        for name, enum_val in mapping.items():
            if self.value == enum_val:
                return name
        return default

    return dunder_str



import click

from sdgs_tools import __version__
from sdgs_tools.cli import aplikasi
from sdgs_tools.cli import dashboard
from sdgs_tools.cli import exporter
from sdgs_tools.gui import MainApp


@click.group("sdgs-tools", invoke_without_command=True)
@click.pass_context
def cli(ctx: click.Context):
    if ctx.invoked_subcommand is None:
        gui = MainApp()
        click.echo("Memulai GUI...")
        gui()


cli.add_command(aplikasi, "aplikasi")
cli.add_command(dashboard, "dashboard")
cli.add_command(exporter, "exporter")


def main():
    click.echo(f"sdgs-tools v{__version__}")
    click.echo("Dibuat oleh Habib Rohman")
    click.echo("Error? pm https://t.me/hexatester")
    click.echo("Open donasi https://saweria.co/hexatester")
    cli()


if __name__ == "__main__":
    main()



from typing import Any, Dict, Optional
from uiautomator2 import Device, UiObject

from sdgs_tools.aplikasi_sdgs.utils import d_get_text, menu_to


# resourceId
FASILITAS_KESEHATAN_COL = {
    # Kesehatan - Fasilitas kesehatan
    "rumah_sakit": "com.kemendes.survey:id/txtRS",
    "rumah_sakit_bersalin": "com.kemendes.survey:id/txtRSBersalin",
    "puskesmas_rawat_inap": "com.kemendes.survey:id/txtPuskesmasRawatInap",
    "puskesmas_tanpa_inap": "com.kemendes.survey:id/txtPuskesmasTanpaRawatInap",
    "puskesmas_pembantu": "com.kemendes.survey:id/txtPuskesmasPembantu",
    "poliklinik": "com.kemendes.survey:id/txtPoliklinik",
    "tempat_praktik_dokter": "com.kemendes.survey:id/txtPraktikDokter",
    "rumah_bersalin": "com.kemendes.survey:id/txtRumahBersalin",
    "tempat_praktik_bidan": "com.kemendes.survey:id/txtPraktikBidan",
    "poskesdes": "com.kemendes.survey:id/txtPoskedes",
    "polindes": "com.kemendes.survey:id/txtPolindes",
    "apotik": "com.kemendes.survey:id/txtApotik",
    "toko_obat_jamu": "com.kemendes.survey:id/txtTokoKhusus",
    "posyandu": "com.kemendes.survey:id/txtPosyandu",
    "posbindu": "com.kemendes.survey:id/txtPosbindu",
    "tempat_praktik_dukun": "com.kemendes.survey:id/txtPraktikDukun",
}


def get_data_fasilitas_kesehatan(d: Device) -> Dict[str, Any]:
    menu_to(d, "KESEHATAN")
    data: Dict[str, Any] = dict()
    # d(text="KESEHATAN").click()
    for name, resourceId in FASILITAS_KESEHATAN_COL.items():
        value = d_get_text(d, resourceId)
        if value != "Jumlah":
            data[name] = value
    d(className="android.widget.ScrollView").fling.vert.backward()
    return data



from enum import Enum
from sdgs_tools.dashboard.utils import make_str_to_enum, make_dunder_str

MAPPING = {
    "Baik": "1",
    "Cukup": "2",
    "Buruk": "3",
}


class KeterbukaanDesa(Enum):
    BAIK = "1"
    CUKUP = "2"
    BURUK = "3"

    from_str = classmethod(make_str_to_enum(MAPPING))
    __str__ = make_dunder_str(MAPPING)



from typing import Any, Dict
from uiautomator2 import Device

from sdgs_tools.aplikasi_sdgs.utils import d_get_text, menu_to


BANTUAN_PEMERINTAH = [
    "blt_dd",
    "pkh",
    "bst",
    "banpres",
    "umkm",
    "bantuan_pekerja",
    "bantuan_pendidikan_anak",
    "lainnya",
]

LAIN_LAIN_COL = {
    "transport_umum": "com.kemendes.survey:id/txtTransportUmumSebulanTerakhir",
    "transport_umum_bulan_sebelumnya": "com.kemendes.survey:id/txtTranportUmumSebelumnya",
    # Bantuan
    "blt_dd": "com.kemendes.survey:id/cbBLT",
    "pkh": "com.kemendes.survey:id/cbPKH",
    "bst": "com.kemendes.survey:id/cbBST",
    "banpres": "com.kemendes.survey:id/cbBanpres",
    "umkm": "com.kemendes.survey:id/cbUMKM",
    "bantuan_pekerja": "com.kemendes.survey:id/cbBantuanPekerja",
    "bantuan_pendidikan_anak": "com.kemendes.survey:id/cbBantuanPendidikan",
    "lainnya": "com.kemendes.survey:id/cbBantuanLainnya",
    # Lain
    "pengeluaran_bulanan": "com.kemendes.survey:id/txtPengeluaranRata2",
}


def get_data_lain_lain(d: Device) -> Dict[str, Any]:
    menu_to(d, "LAIN-LAIN")
    data: Dict[str, Any] = dict()
    for name, resourceId in LAIN_LAIN_COL.items():
        data[name] = d_get_text(d, resourceId)
    bantuan: Dict[str, Any] = dict()
    for name in BANTUAN_PEMERINTAH:
        bantuan[name] = data.pop(name)
    data["penerima_program_pemerintah"] = bantuan
    d(className="android.widget.ScrollView").fling.vert.backward()
    return data



from typing import List, Union


def parse_range(value: Union[str, int]) -> List[int]:
    results: List[int] = list()
    if isinstance(value, int):
        results.append(value)
    elif value.isdigit():
        results.append(int(value))
    elif "," in value:
        for val in value.split(","):
            # For 1,2
            results.extend(parse_range(val))
    elif "-" in value:
        start, end = value.split("-")
        if start.isdigit() and end.isdigit():
            if start == end:
                # For 1-1
                results.append(int(start))
            else:
                # For 1-2
                results.extend(range(int(start), int(end) + 1))
    return results



from enum import Enum
from sdgs_tools.dashboard.utils import make_str_to_enum, make_dunder_str

MAPPING = {
    "Sendiri": "1",
    "Berkelompok/tetangga": "2",
    "MCK umum": "3",
    "Tidak ada": "4",
}


class FasilitasMck(Enum):
    SENDIRI = "1"
    BERKELOMPOK = "2"
    UMUM = "3"
    TIDAK_ADA = "4"

    from_str = classmethod(make_str_to_enum(MAPPING, "1"))
    __str__ = make_dunder_str(MAPPING)



import logging
import tkinter as tk
from tkinter.filedialog import asksaveasfilename
from tkinter.messagebox import showerror, showinfo, showwarning

import adbutils
from uiautomator2.init import Initer

from .export_individu import ExportIndividuWindow
from .export_keluarga import ExportKeluargaWindow


class AplikasiTab(tk.Frame):
    def __init__(self, master: tk.Frame, **kw):
        super().__init__(master, bg="#CCC", width=500, height=500, **kw)
        # self.pack(expand=True, fill="both", side="right")
        self.grid(row=0, column=0, sticky="nsew")
        self.label_info = tk.Label(
            self,
            text="Fitur ini untuk meng eksport data dari aplikasi sdgs android",
        )
        self.label_info.grid(row=0, column=0, columnspan=2)
        self.init_device_button = tk.Button(
            self,
            text="Inisiasi Perangkat Android",
            command=self.init_device,
        )
        self.init_device_button.grid(row=1, column=0, columnspan=2)
        self.eksport_individu_button = tk.Button(
            self,
            text="Eksport Data Individu",
            command=ExportIndividuWindow,
        )
        self.eksport_individu_button.grid(row=2, column=0)
        self.eksport_keluarga_button = tk.Button(
            self,
            text="Eksport Data Keluarga",
            command=ExportKeluargaWindow,
        )
        self.eksport_keluarga_button.grid(row=2, column=1)

    @staticmethod
    def init_device():
        perangkat = 0
        for device in adbutils.adb.iter_device():
            init = Initer(device, loglevel=logging.INFO)
            init.install()
        else:
            showwarning("Gagal", "Tidak ada perangkat untuk diinisiasi")
            return
        showinfo("Berhasil", f"Berhasil menginisiasi {perangkat} perangkat")



from typing import List, Union

from .auth import BaseAuth
from .import_individu import DataIndividu
from .import_keluarga import DataKeluarga


class Sdgs(BaseAuth):
    def __init__(
        self,
        username: str,
        password: str,
        api_server="https://sdgsdev.kemendesa.go.id/",
    ):
        super().__init__(api_server=api_server)
        self.token = self.login(username, password)

    def validateNikKepalaKeluarga(self, nik: str, nokk: str) -> bool:
        json_data = {"nik": nik, "nokk": nokk}
        res = self.api_post_raw(
            "surveyIndividu/validateNik",
            str,
            json=json_data,
            ignore=True,
        )
        return res.message != "NIK Kepala Keluarga Sudah Ada"

    def validateNik(
        self,
        nik: str,
    ) -> bool:
        json_data = {"nik": nik}
        res = self.api_post_raw(
            "surveyIndividu/validateNik",
            str,
            json=json_data,
            ignore=True,
        )
        return res.message != "NIK Sudah Ada"

    def save_individu(self, individu: DataIndividu, rt: str, rw: str):
        data_individu = individu.make_data(desa=self.token.wilayah, rt=rt, rw=rw)
        json_data = {"jsonResult": data_individu}
        res = self.api_post_raw(
            filename="surveyIndividu/save",
            cl=str,
            json=json_data,
        )
        return res

    def save_keluarga(self, keluarga: DataKeluarga, rt: str, rw: str):
        data_keluarga = keluarga.make_data(desa=self.token.wilayah, rt=rt, rw=rw)
        json_data = {"jsonResult": data_keluarga}
        res = self.api_post_raw(
            filename="surveyKeluarga/save",
            cl=str,
            json=json_data,
        )
        return res

    def update_individu(
        self, id_individu: Union[str, int], individu: DataIndividu, rt: str, rw: str
    ):
        data_individu = individu.make_data(desa=self.token.wilayah, rt=rt, rw=rw)
        json_data = {"id": id_individu, "jsonResult": data_individu}
        res = self.api_post_raw(
            filename="surveyIndividu/update",
            cl=str,
            json=json_data,
        )
        return res

    def get_individu(
        self,
        search: str = "",
        page: int = 1,
        pageSize: int = 10,
        kodeDesa: str = None,
        isSortAsc: bool = True,
    ):
        json_data = {
            "search": search,
            "page": page,
            "pageSize": pageSize,
            "kodeDesa": kodeDesa,
            "isSortAsc": isSortAsc,
        }
        res = self.api_post_raw(
            filename="surveyIndividu/getSurveyPagedData",
            cl=str,
            json=json_data,
        )
        return res



from enum import Enum
from sdgs_tools.dashboard.utils import make_str_to_enum, make_dunder_str

MAPPING = {
    "Gas kota/LPG/biogas": "1",
    "Minyak tanah/batu bara": "2",
    "Kayu bakar": "3",
    "Lainnya": "other",
}


class EnergiMemasak(Enum):
    GAS = "1"
    MINYAK_TANAH_BATU_BARA = "2"
    KAYU_BAKAR = "3"
    LAINNYA = "other"

    from_str = classmethod(make_str_to_enum(MAPPING, "1"))
    __str__ = make_dunder_str(MAPPING)



from typing import Any, Dict
from uiautomator2 import Device

from sdgs_tools.aplikasi_sdgs.utils import d_get_text, menu_to

KELUARGA_COL = {
    "nama": "com.kemendes.survey:id/txtNama",
    "alamat": "com.kemendes.survey:id/txtAlamat",
    "nomor_hp": "com.kemendes.survey:id/txtHP",
    "telepon_rumah": "com.kemendes.survey:id/txtNoTelpon",
    "nik": "com.kemendes.survey:id/txtNIKKepalaKeluarga",
    "tempat_tinggal": "com.kemendes.survey:id/cbTempatTinggal",
    "status_lahan": "com.kemendes.survey:id/cbStatusLahan",
    "luas_lantai": "com.kemendes.survey:id/txtLuasLantaiTempatTinggal",
    "luas_lahan": "com.kemendes.survey:id/txtLuasLahanTempatTinggal",
    "lantai": "com.kemendes.survey:id/cbJenisLantai",
    "dinding": "com.kemendes.survey:id/cbDinding",
    "jendela": "com.kemendes.survey:id/cbJendela",
    "atap": "com.kemendes.survey:id/cbAtap",
    "penerangan": "com.kemendes.survey:id/cbPenerangan",
    "energi_memasak": "com.kemendes.survey:id/cbEnergiMemasak",
    "pembuangan_sampah": "com.kemendes.survey:id/cbTempatPembuangan",
    "fasilitas_mck": "com.kemendes.survey:id/cbFasilitasMCK",
    "sumber_air_mandi": "com.kemendes.survey:id/cbSumberAirMandi",
    "fasilitas_bab": "com.kemendes.survey:id/cbFasilitasBuangAir",
    "sumber_air_minum": "com.kemendes.survey:id/cbSumberAirMinum",
    "pembuangan_limbah_cair": "com.kemendes.survey:id/cbTempatPembuanganLimbah",
    "bawah_sutet": "com.kemendes.survey:id/cbSUTET",
    "bantaran_sungai": "com.kemendes.survey:id/cbBantaranSungai",
    "lereng_gunung": "com.kemendes.survey:id/cbRumahDiLereng",
    "kumuh": "com.kemendes.survey:id/cbKondisiRumah",
}


def get_data_lokasi(d: Device) -> Dict[str, Any]:
    menu_to(d, "LOKASI & PEMUKIMAN")
    data: Dict[str, Any] = dict()
    for name, resourceId in KELUARGA_COL.items():
        data[name] = d_get_text(d, resourceId)
    data["luas"] = {
        "lantai": data.pop("luas_lantai"),
        "lahan": data.pop("luas_lahan"),
    }
    d(className="android.widget.ScrollView").fling.vert.backward()
    data["kumuh"] = "Ya" if data["kumuh"] == "Kumuh" else "Tidak"
    return data



import cattr
import click
from openpyxl import load_workbook
from time import sleep
from typing import Any, Dict, List
from uiautomator2 import Device

from sdgs_tools.aplikasi_sdgs.utils import menu_to
from sdgs_tools.dashboard.import_keluarga import DataKeluarga
from sdgs_tools.dashboard.import_individu.mapping import MAPPING_COLS

from .kesehatan import get_data_kesehatan
from .lain_lain import get_data_lain_lain
from .lokasi import get_data_lokasi
from .pendidikan import get_data_pendidikan
from .sarpras import get_data_sarpras
from .tenaga_kesehatan import get_data_tenaga_kesehatan


__all__ = [
    "get_data_kesehatan",
    "get_data_lain_lain",
    "get_data_lokasi",
    "get_data_pendidikan",
    "get_data_sarpras",
    "get_data_tenaga_kesehatan",
]


def export_keluarga(
    d: Device,
    filepath: str,
    rt_rw: str,
    rows: List[int],
    skip_lokasi: bool = False,
    skip_pendidikan: bool = False,
    skip_kesehatan: bool = False,
    skip_tenaga_kesehatan: bool = False,
    skip_sarpras: bool = False,
    skip_lain_lain: bool = False,
):
    wb = load_workbook(filepath)
    keluarga_ws = wb["Keluarga"]
    form_rt_rw = d(resourceId="com.kemendes.survey:id/txtRTRW")
    while not form_rt_rw.exists:
        click.echo(
            "Mohon aktifkan GPS, buka aplikasi, dan masuk menu Entri Survey Keluarga!"
        )
        sleep(1)
    skipped = 0
    failed = 0
    success = 0
    no_kk_col: str = MAPPING_COLS["no_kk"]
    for row in rows:
        no_kk = keluarga_ws[f"{no_kk_col}{row}"].value
        if not no_kk:
            click.echo(f"Kolom nomor kk kosong di baris {row}, membatalkan operasi")
            return False
        form_rt_rw = d(resourceId="com.kemendes.survey:id/txtRTRW")
        form_rt_rw.send_keys(rt_rw)
        form_kk = d(resourceId="com.kemendes.survey:id/txtNoKK")
        form_kk.send_keys(no_kk)
        d.press("back")
        d(resourceId="com.kemendes.survey:id/btnCariRT").click()
        menu_to(d, "LOKASI & PEMUKIMAN")
        d(className="android.widget.ScrollView").fling.vert.backward()
        if not d(resourceId="com.kemendes.survey:id/txtNama").info.get("text"):
            click.echo(f"Lewati data kosong untuk rtrw {rt_rw} no kk {no_kk}")
            return False
        data: Dict[str, Any] = {"no_kk": no_kk}
        if not skip_lokasi:
            data.update(get_data_lokasi(d))
        if not skip_pendidikan:
            data["akses_pendidikan"] = get_data_pendidikan(d)
        if not skip_kesehatan:
            data["akses_fasilitas_kesehatan"] = get_data_kesehatan(d)
        if not skip_tenaga_kesehatan:
            data["akses_tenaga_kesehatan"] = get_data_tenaga_kesehatan(d)
        if not skip_sarpras:
            data["akses_sarpras_transport"] = get_data_sarpras(d)
        if not skip_lain_lain:
            data.update(get_data_lain_lain(d))
        try:
            keluarga: DataKeluarga = cattr.structure(data, DataKeluarga)
            keluarga.save(keluarga_ws, row)
            click.echo(f"Berhasil mengekspor baris {row} : {keluarga}")
            success += 1
        except ValueError as e:
            click.echo(f"Baris {row} dilewati karena : {e}")
            skipped += 1
            continue
        except Exception as e:
            click.echo(f"Error ketika membuat DataKeluarga baris {row} : {e}")
            failed += 1
            continue
    wb.save(filepath)



from enum import Enum
from sdgs_tools.dashboard.utils import make_str_to_enum, make_dunder_str

MAPPING = {
    "Cepat": "1",
    "Sedang": "2",
    "Lambat": "3",
}


class KecepatanInternet(Enum):
    CEPAT = "1"
    SEDANG = "2"
    LAMBAT = "3"

    from_str = classmethod(make_str_to_enum(MAPPING))
    __str__ = make_dunder_str(MAPPING)



import cattr
import click
from openpyxl import load_workbook
from typing import List, Optional, TYPE_CHECKING

if TYPE_CHECKING:
    from sdgs_tools.dashboard.sdgs import Sdgs

from .akses import Akses
from .sarpras_transport import SarprasTransport

from .akses_fasilitas_kesehatan import AksesFasilitasKesehatan
from .akses_pendidikan import AksesPendidikan
from .akses_sarpras_transport import AksesSarprasTransport
from .akses_tenaga_kesehatan import AksesTenagaKesehatan
from .bantuan_pemerintah import BantuanPemerintah
from .luas import Luas

from .data_keluarga import DataKeluarga

cattr.register_unstructure_hook(AksesFasilitasKesehatan, AksesFasilitasKesehatan.todict)
cattr.register_unstructure_hook(AksesPendidikan, AksesPendidikan.todict)
cattr.register_unstructure_hook(AksesSarprasTransport, AksesSarprasTransport.todict)
cattr.register_unstructure_hook(AksesTenagaKesehatan, AksesTenagaKesehatan.todict)
cattr.register_unstructure_hook(Akses, Akses.todict)
cattr.register_unstructure_hook(BantuanPemerintah, BantuanPemerintah.todict)
cattr.register_unstructure_hook(AksesTenagaKesehatan, AksesTenagaKesehatan.todict)

cattr.register_structure_hook(BantuanPemerintah, BantuanPemerintah.from_str)

__all__ = [
    "AksesFasilitasKesehatan",
    "AksesPendidikan",
    "AksesSarprasTransport",
    "AksesTenagaKesehatan",
    "Akses",
    "BantuanPemerintah",
    "DataKeluarga",
    "Luas",
    "SarprasTransport",
]


def import_keluarga(
    sdgs: "Sdgs",
    filepath: str,
    rows: List[int],
    rt: str,
    rw: str,
):
    click.echo(f"Membuka {filepath}")
    wb = load_workbook(filepath, read_only=True)
    keluarga = wb["Keluarga"]
    faileds: List[str] = list()
    failed = 0
    trying = 0
    success = 0
    skipped = 0
    for row in rows:
        if row < 4:
            raise ValueError(f"Tidak dapat memproses baris < 4")
        no_kk, nik = DataKeluarga.get_kk_nik(keluarga, row)
        if not no_kk:
            raise ValueError(f"Nomor KK kosong di baris {row}. Membatalkan operasi!")
        elif not nik:
            raise ValueError(
                f"NIK kepala keluarga kosong di baris {row}. Membatalkan operasi!"
            )
        if not sdgs.token.token.is_valid():
            sdgs.token = sdgs.token_refresh(sdgs.token)
        if not sdgs.validateNikKepalaKeluarga(nik, no_kk):
            click.echo(f"Melewati KK {no_kk}, karena data sudah diinput")
            skipped += 1
            continue
        click.echo(f"Mempersiapkan data keluarga {no_kk}")
        data = DataKeluarga.make(keluarga, row)
        try:
            data_keluarga: DataKeluarga = cattr.structure(data, DataKeluarga)
        except ValueError as e:
            click.echo(f"Baris {row} dilewati karena : {e}")
            skipped += 1
            continue
        except Exception as e:
            click.echo(f"Eror ketika membuat DataKeluarga baris {row}")
            failed += 1
            continue
        if not sdgs.token.token.is_valid():
            sdgs.token = sdgs.token_refresh(sdgs.token)
        try:
            click.echo(f"Berhasil mengirim data {data_keluarga}")
            res = sdgs.save_keluarga(data_keluarga, rt, rw)
            if res:
                click.echo(f"{row}. {data_keluarga} : {res.message}")
                success += 1
            elif res.message:
                click.echo(f"Gagal mengirim data {data_keluarga} : {res.message}")
            else:
                click.echo(f"Gagal mengirim data {data_keluarga}")
        except Exception as e:
            faileds.append(str(row))
            click.echo(f"Gagal menyimpan keluarga {no_kk} karena {e}")
            failed += 1
    click.echo(
        f"Berhasil : {success}\nMencoba : {trying}\n"
        f"Dilewati : {skipped}\nError : {failed}"
    )
    if faileds:
        click.echo("Baris yang gagal : " + ",".join(faileds))



from enum import Enum
from sdgs_tools.dashboard.utils import make_str_to_enum, make_dunder_str

MAPPING = {
    "Tidak ada": 1,
    "Di kebun/sungai/drainase": 2,
    "Dibakar": 3,
    "Tempat sampah": 4,
    "Tempat sampah diangkut reguler": 5,
}


class PembuanganSampah(Enum):
    TIDAK_ADA = 1
    KEBUN_SUNGAI_DRAINASE = 2
    DIBAKAR = 3
    TEMPAT_SAMPAH = 4
    TEMPAT_SAMPAH_DIANGKUT = 5

    from_str = classmethod(make_str_to_enum(MAPPING, 1))
    __str__ = make_dunder_str(MAPPING)



from enum import Enum
from sdgs_tools.dashboard.utils import make_str_to_enum, make_dunder_str

MAPPING = {
    "Ya": "1",
    "Tidak": "2",
}


class YaTidak(Enum):
    YA = "1"
    TIDAK = "2"

    def __bool__(self):
        return self.value == self.YA

    from_str = classmethod(make_str_to_enum(MAPPING, "2"))
    __str__ = make_dunder_str(MAPPING)



from enum import Enum
from sdgs_tools.dashboard.utils import make_str_to_enum, make_dunder_str

MAPPING = {
    "Islam": "1",
    "Kristen": "2",
    "Katolik": "3",
    "Hindu": "4",
    "Budha": "5",
    "Khonghucu": "6",
    "Lainnya": "other",
}


class Agama(Enum):
    ISLAM = "1"
    KRISTEN = "2"
    KATOLIK = "3"
    HINDU = "4"
    BUDHA = "5"
    KHONGHUCU = "6"
    LAINNYA = "other"

    from_str = classmethod(make_str_to_enum(MAPPING))
    __str__ = make_dunder_str(MAPPING)



import cattr
import click
from openpyxl import load_workbook
from openpyxl.worksheet.worksheet import Worksheet
from time import sleep
from typing import Any, Dict, List, Tuple
from uiautomator2 import Device

from sdgs_tools.aplikasi_sdgs.utils import menu_to
from sdgs_tools.dashboard.import_individu import DataIndividu
from sdgs_tools.dashboard.import_individu.mapping import MAPPING_COLS

from .disabilitas import get_data_disabilitas
from .individu import get_data_individu
from .kesehatan import get_data_kesehatan
from .pekerjaan import get_data_pekerjaan
from .pendidikan import get_data_pendidikan
from .penghasilan import get_data_penghasilan


__all__ = [
    "get_data_disabilitas",
    "get_data_individu",
    "get_data_kesehatan",
    "get_data_pekerjaan",
    "get_data_pendidikan",
    "get_data_penghasilan",
]


def get_param(
    ws: Worksheet, row: int, kk: str = "no_kk", nik: str = "nik"
) -> Tuple[str, str]:
    kk_col = MAPPING_COLS[kk]
    nik_col = MAPPING_COLS[nik]
    return (ws[f"{kk_col}{row}"].value, ws[f"{nik_col}{row}"].value)


def export_individu(
    d: Device,
    filepath: str,
    rt_rw: str,
    rows: List[int],
    start_row_penghasilan: int = 2,
    skip_individu: bool = False,
    skip_pekerjaan: bool = False,
    skip_penghasilan: bool = False,
    skip_kesehatan: bool = False,
    skip_disabilitas: bool = False,
    skip_pendidikan: bool = False,
):
    wb = load_workbook(filepath)
    individu_ws = wb["Individu"]
    form_rt_rw = d(resourceId="com.kemendes.survey:id/txtRTRW")
    while not form_rt_rw.exists:
        click.echo(
            "Mohon aktifkan GPS, buka aplikasi, dan masuk menu Entri Survey Individu!"
        )
        sleep(1)
    skipped = 0
    failed = 0
    success = 0
    row_penghasilan = start_row_penghasilan
    for row in rows:
        no_kk, nik = get_param(individu_ws, row)
        # Input rt rw no_kk nik & Tampilkan
        form_rt_rw = d(resourceId="com.kemendes.survey:id/txtRTRW")
        form_rt_rw.send_keys(rt_rw)
        form_kk = d(resourceId="com.kemendes.survey:id/txtNoKK")
        form_kk.send_keys(no_kk)
        form_nik = d(resourceId="com.kemendes.survey:id/txtNIK")
        form_nik.send_keys(nik)
        d.press("back")
        d(resourceId="com.kemendes.survey:id/btnCariRT").click()
        menu_to(d, "DATA INDIVIDU")
        # Get Data
        data: Dict[str, Any] = {"no_kk": no_kk, "nik": nik}
        # Individu
        if not skip_individu:
            data.update(get_data_individu(d))
        # Pekerjaan
        if not skip_pekerjaan:
            data.update(get_data_pekerjaan(d))
        # Penghasilan
        if not skip_penghasilan:
            data["penghasilan"] = get_data_penghasilan(d)
        # Kesehatan
        if not skip_kesehatan:
            data.update(get_data_kesehatan(d))
        # Disabilitas
        if not skip_disabilitas:
            data["disabilitas"] = get_data_disabilitas(d)
        # Pendidikan
        if not skip_pendidikan:
            data.update(get_data_pendidikan(d))
        try:
            individu: DataIndividu = cattr.structure(data, DataIndividu)
            row_penghasilan = individu.save(wb, row, row_penghasilan)
            click.echo(f"Berhasil mengekspor baris {row} : {individu}")
            success += 1
        except ValueError as e:
            click.echo(f"Baris {row} dilewati karena : {e}")
            skipped += 1
            continue
        except Exception as e:
            click.echo(f"Error ketika membuat DataIndividu baris {row} : {e}")
            failed += 1
            continue
    wb.save(filepath)



import tkinter as tk
from typing import Callable, Dict


class SidebarFrame(tk.Frame):
    def __init__(self, master, **kw):
        super().__init__(
            master,
            width=200,
            height=500,
            bg="white",
            relief="sunken",
            borderwidth=2,
            **kw
        )
        self.pack(expand=False, fill="both", side="left", anchor="nw")
        self.title = tk.Label(self, text="SDGS Tools")
        self.title.pack()
        self.buttons: Dict[str, tk.Button] = dict()

    def add_button(self, text: str, command: Callable[[], None]):
        button = tk.Button(self, text=text, command=command)
        self.buttons[text] = button
        button.pack()



import attr
import cattr
from enum import Enum
from openpyxl import Workbook
from openpyxl.worksheet.worksheet import Worksheet
from typing import Any, Dict, Optional, Tuple

from . import (
    Luas,
    AksesFasilitasKesehatan,
    AksesPendidikan,
    AksesSarprasTransport,
    AksesTenagaKesehatan,
    BantuanPemerintah,
)
from .enums import (
    Atap,
    Dinding,
    EnergiMemasak,
    FasilitasBab,
    FasilitasMck,
    Jendela,
    Lantai,
    PembuanganLimbahCair,
    PembuanganSampah,
    Penerangan,
    StatusLahan,
    SumberAirMandi,
    SumberAirMinum,
    SumberKayuBakar,
    TempatTinggal,
    YaTidak,
)
from .mapping import MAPPING, MAPPING_COLS


@attr.dataclass(kw_only=True)
class DataKeluarga:
    # DESKRIPSI KELUARGA & PEMUKIMAN
    no_kk: str
    nik: str
    nama: str
    alamat: str
    nomor_hp: str
    telepon_rumah: str
    anggota_keluarga: Optional[str] = None
    tempat_tinggal: TempatTinggal
    tempat_tinggal_comment: Optional[str] = None
    status_lahan: StatusLahan
    status_lahan_comment: Optional[str] = None
    luas: Luas
    lantai: Lantai
    lantai_other: Optional[str] = None
    dinding: Dinding
    dinding_other: Optional[str] = None
    jendela: Jendela
    atap: Atap
    atap_comment: Optional[str] = None
    penerangan: Penerangan
    energi_memasak: EnergiMemasak
    sumber_kayu_bakar: Optional[SumberKayuBakar] = None
    pembuangan_sampah: PembuanganSampah
    fasilitas_mck: FasilitasMck
    sumber_air_mandi: SumberAirMandi
    sumber_air_mandi_comment: Optional[str] = None
    fasilitas_bab: FasilitasBab
    fasilitas_bab_comment: Optional[str] = None
    sumber_air_minum: SumberAirMinum
    sumber_air_minum_comment: Optional[str] = None
    pembuangan_limbah_cair: PembuanganLimbahCair
    pembuangan_limbah_cair_comment: Optional[str] = None
    bawah_sutet: YaTidak
    bantaran_sungai: YaTidak
    lereng_gunung: YaTidak
    kumuh: YaTidak
    # AKSES PENDIDIKAN TERDEKAT
    akses_pendidikan: AksesPendidikan
    # AKSES FASILITAS KESEHATAN TERDEKAT
    akses_fasilitas_kesehatan: AksesFasilitasKesehatan
    # AKSES TENAGA KESEHATAN TERDEKAT
    akses_tenaga_kesehatan: AksesTenagaKesehatan
    # AKSES PRASARANA DAN SARANA TRANSPORTASI
    akses_sarpras_transport: AksesSarprasTransport
    # LAIN-LAIN
    transport_umum: str = "0"
    transport_umum_bulan_sebelumnya: str = "0"
    penerima_program_pemerintah: BantuanPemerintah
    pengeluaran_bulanan: str = "0"

    def __str__(self):
        return f"{self.nama} ({self.no_kk})"

    def __attrs_post_init__(self) -> None:
        if self.no_kk in (None, "None"):
            raise ValueError("Nomor KK harus diisi")
        elif len(self.no_kk) != 16:
            raise ValueError("Nomor KK harus 16 digit")
        if self.nik in (None, "None"):
            raise ValueError("NIK Kepala Keluarga harus diisi")
        elif len(self.nik) != 16:
            raise ValueError("NIK Kepala Keluarga harus 16 digit")
        if self.nama in (None, "None"):
            raise ValueError("Nama keluarga harus diisi")
        if (
            self.tempat_tinggal == TempatTinggal.LAINNYA
            and not self.tempat_tinggal_comment
        ):
            raise ValueError("Tempat Tinggal Lainnya harus diisi")
        if self.status_lahan == StatusLahan.LAINNYA and not self.status_lahan_comment:
            raise ValueError("Status Lahan Lainnya harus diisi")
        if self.lantai == Lantai.LAINNYA and not self.lantai_other:
            raise ValueError("Lantai Lainnya harus diisi")
        if self.dinding == Dinding.LAINNYA and not self.dinding_other:
            raise ValueError("Dinding Lainnya harus diisi")
        if self.atap == Atap.LAINNYA and not self.atap_comment:
            raise ValueError("Atap Lainnya harus diisi")
        if (
            self.sumber_air_mandi == SumberAirMandi.LAINNYA
            and not self.sumber_air_mandi_comment
        ):
            raise ValueError("Sumber Air Mandi Lainnya harus diisi")
        if (
            self.fasilitas_bab == FasilitasBab.LAINNYA
            and not self.fasilitas_bab_comment
        ):
            raise ValueError("Fasilitas BAB Lainnya harus diisi")
        if (
            self.sumber_air_minum == SumberAirMinum.LAINNYA
            and not self.sumber_air_minum_comment
        ):
            raise ValueError("Sumber Air Minum Lainnya harus diisi")
        if (
            self.pembuangan_limbah_cair == PembuanganLimbahCair.LAINNYA
            and not self.pembuangan_limbah_cair_comment
        ):
            raise ValueError("Pembuangan Limbah Cair Lainnya harus diisi")

    @staticmethod
    def make(ws: Worksheet, row: int):
        data: Dict[str, Any] = dict()
        for name, col in MAPPING_COLS.items():
            data[name] = ws[f"{col}{row}"].value
        data["luas"] = Luas.make(ws, row)
        data["akses_pendidikan"] = AksesPendidikan.make(ws, row)
        data["akses_fasilitas_kesehatan"] = AksesFasilitasKesehatan.make(ws, row)
        data["akses_tenaga_kesehatan"] = AksesTenagaKesehatan.make(ws, row)
        data["akses_sarpras_transport"] = AksesSarprasTransport.make(ws, row)
        return data

    @staticmethod
    def get_kk_nik(ws: Worksheet, row: int) -> Tuple[str, str]:
        return (
            ws[f"{MAPPING_COLS['no_kk']}{row}"].value,
            ws[f"{MAPPING_COLS['nik']}{row}"].value,
        )

    def make_data(self, desa: str, rt: str, rw: str):
        raw_data: Dict[str, Any] = cattr.unstructure(self)
        clean_data: Dict[str, Any] = dict()
        for key, properti in MAPPING.items():
            value = raw_data.get(properti)
            if value in (None, "None"):
                # TODO Improve empty value detection
                continue
            clean_data[key] = value
        clean_data.update(self.akses_pendidikan.todict())
        clean_data.update(self.akses_tenaga_kesehatan.todict())
        clean_data.update(self.akses_fasilitas_kesehatan.todict())
        clean_data.update(self.akses_sarpras_transport.todict())
        clean_data.update(
            {
                "desa": desa,
                "kecamatan": desa[0:7],
                "kota": desa[0:4],
                "provinsi": desa[0:2],
                "rt": rt,
                "rw": rw,
            }
        )
        return clean_data

    def save(self, ws: Worksheet, row: int):
        MAPPING = dict(MAPPING_COLS)
        MAPPING.pop("penerima_program_pemerintah")
        for name, col in MAPPING.items():
            value = getattr(self, name)
            if value is None:
                continue
            if isinstance(value, Enum):
                value = str(value)
            ws[f"{col}{row}"] = value
        # Luas
        ws[f"L{row}"] = self.luas.lantai
        ws[f"M{row}"] = self.luas.lahan
        self.akses_pendidikan.save(ws, row)
        self.akses_fasilitas_kesehatan.save(ws, row)
        self.akses_tenaga_kesehatan.save(ws, row)
        self.akses_sarpras_transport.save(ws, row)
        ws[f"EL{row}"] = str(self.penerima_program_pemerintah)



import click
import requests


TEMPLATE_INDIVIDU = "https://hexatester.github.io/sdgs-tools/Template%20Individu%20Dashboard%20SDGS%20Kemendesa.xlsm"
TEMPLATE_KELUARGA = "https://hexatester.github.io/sdgs-tools/Template%20Keluarga%20Dashboard%20SDGS%20Kemendesa.xlsm"


@click.command("template-individu")
@click.argument(
    "nama-file",
    type=click.Path(exists=False),
    default="Template-Individu-Dashboard.xlsm",
)
def template_individu(nama_file: str):
    res = requests.get(TEMPLATE_INDIVIDU)
    with open(nama_file, "wb") as f:
        f.write(res.content)
    click.echo(f"Berhasil menyimpan template di {nama_file}")


@click.command("template-keluarga")
@click.argument(
    "nama-file",
    type=click.Path(exists=False),
    default="Template-Keluarga-Dashboard.xlsm",
)
def template_keluarga(nama_file: str):
    res = requests.get(TEMPLATE_KELUARGA)
    with open(nama_file, "wb") as f:
        f.write(res.content)
    click.echo(f"Berhasil menyimpan template di {nama_file}")



from requests import Response, Session
from typing import Type, TypeVar

from .response import SdgsResponse
from .utils import register_cattr_hooks

T = TypeVar("T")


class BaseSdgs:
    def __init__(self, api_server: str = "https://sdgsdev.kemendesa.go.id/"):
        self.api_server = api_server
        self.session = Session()
        register_cattr_hooks()

    def url_api(self, filename: str) -> str:
        return self.api_server + filename.lstrip("/")

    def api_get(self, filename: str, *args, **kwargs) -> Response:
        return self.session.get(self.url_api(filename), *args, **kwargs)

    def api_post(self, filename: str, *args, **kwargs) -> Response:
        headers = {
            "Sec-Fetch-Dest": "empty",
            "Sec-Fetch-Mode": "cors",
            "Sec-Fetch-Site": "same-site",
        }
        if "headers" in kwargs:
            headers.update(kwargs["headers"])
        kwargs["headers"] = headers
        return self.session.post(
            self.url_api(filename),
            *args,
            **kwargs,
        )

    def api_get_to_res(self, filename: str, cl: Type[T], *args, **kwargs) -> T:
        res_raw = self.api_get(filename, *args, **kwargs)
        res = SdgsResponse.from_str(res_raw.text, cl)
        return res.data

    def api_post_to_res(self, filename: str, cl: Type[T], *args, **kwargs) -> T:
        res_raw = self.api_post(filename, *args, **kwargs)
        res = SdgsResponse.from_str(res_raw.text, cl)
        return res.data

    def api_post_raw(
        self,
        filename: str,
        cl: Type[T],
        ignore: bool = True,
        *args,
        **kwargs,
    ) -> SdgsResponse[T]:
        res_raw = self.api_post(filename, *args, **kwargs)
        return SdgsResponse.from_str(res_raw.text, cl, ignore)



import cattr

from .agama import Agama
from .akses_internet import AksesInternet
from .diekspor import Diekspor
from .jenis_kelamin import JenisKelamin
from .kecepatan_internet import KecepatanInternet
from .keterbukaan_desa import KeterbukaanDesa
from .kondisi_pekerjaan import KondisiPekerjaan
from .pekerjaan_utama import PekerjaanUtama
from .pelayanan_desa import PelayananDesa
from .pendidikan import Pendidikan
from .status_pernikahan import StatusPernikahan
from .sumber_penghasilan import SumberPenghasilan
from .warganegara import Warganegara
from .ya_tidak import YaTidak


cattr.register_structure_hook(Agama, Agama.from_str)  # type: ignore
cattr.register_structure_hook(AksesInternet, AksesInternet.from_str)  # type: ignore
cattr.register_structure_hook(Diekspor, Diekspor.from_str)  # type: ignore
cattr.register_structure_hook(JenisKelamin, JenisKelamin.from_str)  # type: ignore
cattr.register_structure_hook(KecepatanInternet, KecepatanInternet.from_str)  # type: ignore
cattr.register_structure_hook(KeterbukaanDesa, KeterbukaanDesa.from_str)  # type: ignore
cattr.register_structure_hook(KondisiPekerjaan, KondisiPekerjaan.from_str)  # type: ignore
cattr.register_structure_hook(PekerjaanUtama, PekerjaanUtama.from_str)  # type: ignore
cattr.register_structure_hook(PelayananDesa, PelayananDesa.from_str)  # type: ignore
cattr.register_structure_hook(Pendidikan, Pendidikan.from_str)  # type: ignore
cattr.register_structure_hook(StatusPernikahan, StatusPernikahan.from_str)  # type: ignore
cattr.register_structure_hook(SumberPenghasilan, SumberPenghasilan.from_str)  # type: ignore
cattr.register_structure_hook(Warganegara, Warganegara.from_str)  # type: ignore
cattr.register_structure_hook(YaTidak, YaTidak.from_str)  # type: ignore

__all__ = [
    "Agama",
    "AksesInternet",
    "Diekspor",
    "JenisKelamin",
    "KecepatanInternet",
    "KondisiPekerjaan",
    "PekerjaanUtama",
    "PelayananDesa",
    "Pendidikan",
    "StatusPernikahan",
    "SumberPenghasilan",
    "Warganegara",
    "YaTidak",
]



from typing import Dict


MAPPING: Dict[str, str] = {
    "desa": "desa",
    "I.P103": "nama",
    "I.P104": "jenis_kelamin",
    "I.P105": "tempat_lahir",
    "I.P106": "tanggal_lahir",
    "I.P107": "usia",
    "I.P108": "status_pernikahan",
    "I.P109": "usia_menikah",
    "I.P110": "agama",
    "I.P111": "suku_bangsa",
    "I.P112": "warga_negara",
    "I.P113": "nomor_hp",
    "I.P114": "nomor_whatsapp",
    "I.P115": "alamat_email",
    "I.P116": "alamat_facebook",
    "I.P117": "alamat_twitter",
    "I.P118": "alamat_instagram",
    "I.P119": "aktif_internet",
    "I.P120": "akses_melalui",
    "I.P121": "kecepatan_internet",
    "I.P201": "kondisi_pekerjaan",
    "I.P202": "pekerjaan_utama",
    "I.P202-Comment": "pekerjaan_utama_comment",
    "I.P203": "jamsos_ketenagakerjaan",
    "I.P204": "penghasilan",
    "I.P204_penghasilan": "pekerjaan_penghasilan",
    "I.P401": "penyakit_diderita",
    "I.P402": "fasilitas_kesehatan",
    "I.P403": "jamsos_kesehatan",
    "I.P404": "disabilitas",
    "I.P405": "setahun_melahirkan",
    "I.P406": "mendapat_asi",
    "I.P501": "pendidikan_tertinggi",
    "I.P502": "tahun_pendidikan",
    "I.P503": "pendidikan_diikuti",
    "I.P504": "pelatihan_diikuti",
    "I.P505": "bahasa_permukiman",
    "I.P506": "bahasa_formal",
    "I.P507": "kerja_bakti",
    "I.P508": "siskamling",
    "I.P509": "pesta_rakyat",
    "I.P510": "menolong_kematian",
    "I.P511": "menolong_sakit",
    "I.P512": "menolong_kecelakaan",
    "I.P513": "memperoleh_pelayanan_desa",
    "I.P514": "pelayanan_desa",
    "I.P515": "saran_desa",
    "I.P516": "keterbukaan_desa",
    "I.P517": "terjadi_bencana",
    "I.P518": "terdampak_bencana",
    "kecamatan": "kecamatan",
    "kota": "kota",
    "nik": "nik",
    "no_kk": "no_kk",
    "provinsi": "provinsi",
    "rt": "rt",
    "rw": "rw",
}

MAPPING_COLS: Dict[str, str] = {
    "no_kk": "A",
    "nik": "B",
    "nama": "C",
    "jenis_kelamin": "D",
    "tempat_lahir": "E",
    "tanggal_lahir": "F",
    "usia": "G",
    "status_pernikahan": "H",
    "usia_menikah": "I",
    "agama": "J",
    "suku_bangsa": "K",
    "warga_negara": "L",
    "nomor_hp": "M",
    "nomor_whatsapp": "N",
    "alamat_email": "O",
    "alamat_facebook": "P",
    "alamat_twitter": "Q",
    "alamat_instagram": "R",
    "aktif_internet": "S",
    "akses_melalui": "T",
    "kecepatan_internet": "U",
    "kondisi_pekerjaan": "V",
    "pekerjaan_utama": "W",
    "pekerjaan_utama_comment": "X",
    "jamsos_ketenagakerjaan": "Y",
    # penghasilan: str = "Z"
    "pekerjaan_penghasilan": "AA",
    "penyakit_diderita": "AB",
    # fasilitas_kesehatan: str = "AC-AR"
    "jamsos_kesehatan": "AS",
    # disabilitas: str = "AT"
    "setahun_melahirkan": "AU",
    "mendapat_asi": "AV",
    "pendidikan_tertinggi": "AW",
    "tahun_pendidikan": "AX",
    "pendidikan_diikuti": "AY",
    "pelatihan_diikuti": "AZ",
    "bahasa_permukiman": "BA",
    "bahasa_formal": "BB",
    "kerja_bakti": "BC",
    "siskamling": "BD",
    "pesta_rakyat": "BE",
    "menolong_kematian": "BF",
    "menolong_sakit": "BG",
    "menolong_kecelakaan": "BH",
    "memperoleh_pelayanan_desa": "BI",
    "pelayanan_desa": "BJ",
    "saran_desa": "BK",
    "keterbukaan_desa": "BL",
    "terjadi_bencana": "BM",
    "terdampak_bencana": "BN",
}



from typing import Any, Dict
from uiautomator2 import Device, UiObject

from sdgs_tools.aplikasi_sdgs.utils import d_get_text, menu_to, swipe_box

AKSES_TENKES = {
    "Dokter Spesialis": "spesialis",
    "Dokter Umum": "dokter",
    "Bidan": "bidan",
    "Tenaga Kesehatan": "nakes",
    "Dukun": "dukun",
}

TENAGA_KESEHATAN_COL = {
    # "B": ("com.kemendes.survey:id/txtNIK", ""),
    "nama": ("com.kemendes.survey:id/txtNIK", "", ""),
    "jarak": ("com.kemendes.survey:id/txtNama", "Jarak : ", " km"),
    "waktu": ("com.kemendes.survey:id/txtAlamat", "Waktu Tempuh :", " jam"),
    "kemudahan": ("com.kemendes.survey:id/txtTelpon", "Kemudahan :", ""),
    # "F": ("com.kemendes.survey:id/txtStatus", ""),
}


def get_data_tenaga_kesehatan(d: Device):
    menu_to(d, "AKSES TENAGA KESEHATAN")
    tenkes: Dict[str, Dict[str, Any]] = dict()
    box_daftar_tenaga_kesehatan: UiObject = d(
        resourceId="com.kemendes.survey:id/itemsTenagaKesehatan"
    )
    survey_boxes: UiObject = box_daftar_tenaga_kesehatan.child(
        resourceId="com.kemendes.survey:id/box"
    )
    for survey_box in survey_boxes:
        data: Dict[str, Any] = dict()
        for name, (resourceId, lstrp, rstrp) in TENAGA_KESEHATAN_COL.items():
            current = survey_box.child(resourceId=resourceId)
            value: str = current.info.get("text")
            if isinstance(value, str):
                value = value.lstrip(lstrp)
                data[name] = value.rstrip(rstrp)
        swipe_box(d, survey_box)
        key: str = data.pop("nama")
        tenkes[AKSES_TENKES[key]] = data
    d(className="android.widget.ScrollView").fling.vert.backward()
    return tenkes



from enum import Enum
from sdgs_tools.dashboard.utils import make_str_to_enum, make_dunder_str

MAPPING = {
    "Padi": "1",
    "Palawija": "2",
    "Hortikultura": "3",
    "Karet": "4",
    "Kelapa sawit": "5",
    "Kopi": "6",
    "Kakao": "7",
    "Kelapa": "8",
    "Lada": "9",
    "Cengkeh": "10",
    "Tembakau": "11",
    "Tebu": "12",
    "Sapi potong": "13",
    "Susu sapi": "14",
    "Domba": "15",
    "Ternak besar lainnya": "16",
    "Ayam pedaging": "17",
    "Telur ayam": "18",
    "Ternak kecil lainnya": "19",
    "Perikanan tangkap": "20",
    "Perikanan budidaya": "21",
    "Bambu": "22",
    "Budidaya tanaman kehutanan": "23",
    "Pemungutan hasil hutan": "24",
    "Penangkapan satwa liar": "25",
    "Penangkaran satwa liar": "26",
    "Jasa pertanian": "27",
    "Pertambangan dan penggalian": "28",
    "Industri kerajinan": "29",
    "Industri pengolahan": "30",
    "Perdagangan": "31",
    "Warung dan rumah makan": "32",
    "Angkutan": "33",
    "Pergudangan": "34",
    "Komunikasi": "35",
    "Jasa di luar pertanian": "36",
    "Uang Pensiunan": "37",
    "Karyawan tetap": "38",
    "Karyawan tidak tetap": "39",
    "TNI": "40",
    "PNS": "41",
    "TKI di luar negeri": "42",
    "Sumbangan": "43",
    "Lainnya": "other",
}


class SumberPenghasilan(Enum):
    PADI = "1"
    PALAWIJA = "2"
    HORTIKULTURA = "3"
    KARET = "4"
    KELAPA_SAWIT = "5"
    KOPI = "6"
    KAKAO = "7"
    KELAPA = "8"
    LADA = "9"
    CENGKEH = "10"
    TEMBAKAU = "11"
    TEBU = "12"
    SAPI_POTONG = "13"
    SUSU_SAPI = "14"
    DOMBA = "15"
    TERNAK_BESAR_LAINNYA = "16"
    AYAM_PEDAGING = "17"
    TELUR_AYAM = "18"
    TERNAK_KECIL_LAINNYA = "19"
    PERIKANAN_TANGKAP = "20"
    PERIKANAN_BUDIDAYA = "21"
    BAMBU = "22"
    BUDIDAYA_TANAMAN_KEHUTANAN = "23"
    PEMUNGUTAN_HASIL_HUTAN = "24"
    PENANGKAPAN_SATWA_LIAR = "25"
    PENANGKARAN_SATWA_LIAR = "26"
    JASA_PERTANIAN = "27"
    PERTAMBANGAN_DAN_PENGGALIAN = "28"
    INDUSTRI_KERAJINAN = "29"
    INDUSTRI_PENGOLAHAN = "30"
    PERDAGANGAN = "31"
    WARUNG_DAN_RUMAH_MAKAN = "32"
    ANGKUTAN = "33"
    PERGUDANGAN = "34"
    KOMUNIKASI = "35"
    JASA_DI_LUAR_PERTANIAN = "36"
    UANG_PENSIUNAN = "37"
    KARYAWAN_TETAP = "38"
    KARYAWAN_TIDAK_TETAP = "39"
    TNI = "40"
    PNS = "41"
    TKI_DI_LUAR_NEGERI = "42"
    SUMBANGAN = "43"
    LAINNYA = "other"

    from_str = classmethod(make_str_to_enum(MAPPING, "43"))
    __str__ = make_dunder_str(MAPPING)



from uiautomator2 import Device
from typing import Any, Dict

from sdgs_tools.aplikasi_sdgs.utils import d_get_text, menu_to

DISABILITAS_COL = {
    "tunanetra": "com.kemendes.survey:id/cbTunanetra",
    "tunarungu": "com.kemendes.survey:id/cbTunarungu",
    "tunawicara": "com.kemendes.survey:id/cbTunawicara",
    "tunarungu_wicara": "com.kemendes.survey:id/cbTunarungiwicara",
    "tunadaksa": "com.kemendes.survey:id/cbTunadaksa",
    "tunagrahita": "com.kemendes.survey:id/cbTunagrahita",
    "tunalaras": "com.kemendes.survey:id/cbTunalaras",
    "cacat_eks_kusta": "com.kemendes.survey:id/cbCacat_eks_kusta",
    "cacat_ganda": "com.kemendes.survey:id/cbCacat_ganda",
    "dipasung": "com.kemendes.survey:id/cbDipasung",
}


def get_data_disabilitas(d: Device) -> Dict[str, Any]:
    menu_to(d, "DISABILITAS")
    data: Dict[str, Any] = dict()
    for name, resourceId in DISABILITAS_COL.items():
        data[name] = d_get_text(d, resourceId)
    d(className="android.widget.ScrollView").fling.vert.backward()
    return data



from enum import Enum
from sdgs_tools.dashboard.utils import make_str_to_enum, make_dunder_str

MAPPING = {
    "Listrik PLN": 1,
    "Listrik non PLN": 2,
    "Lampu minyak/lilin": 3,
    "Lainnya": 4,
    "Tidak ada": 5,
}


class Penerangan(Enum):
    LISTRIK_PLN = 1
    LISTRIK_NON_PLN = 2
    LAMPU_MINYAK_LILIN = 3
    LAINNYA = 4
    TIDAK_ADA = 5

    from_str = classmethod(make_str_to_enum(MAPPING, 1))
    __str__ = make_dunder_str(MAPPING)



from enum import Enum
from sdgs_tools.dashboard.utils import make_str_to_enum, make_dunder_str


MAPPING = {
    "Milik Sendiri": "1",
    "Kontrak Sewa": "2",
    "Bebas Sewa": "3",
    "Dipinjami": "4",
    "Dinas": "5",
    "Lainnya": "other",
}


class TempatTinggal(Enum):
    MILIK_SENDIRI = "1"
    KONTRAK_SEWA = "2"
    BEBAS_SEWA = "3"
    DIPINJAMI = "4"
    DINAS = "5"
    LAINNYA = "other"

    from_str = classmethod(make_str_to_enum(MAPPING, "1"))
    __str__ = make_dunder_str(MAPPING)



import cattr
import click
from openpyxl import load_workbook
from typing import List, Optional, TYPE_CHECKING

if TYPE_CHECKING:
    from sdgs_tools.dashboard.sdgs import Sdgs


from .disabilitas import Disabilitas
from .fasilitas_kesehatan import FasilitasKesehatan
from .penghasilan import Penghasilan
from .penyakit_diderita import PenyakitDiderita

from .data_individu import DataIndividu


cattr.register_structure_hook(Disabilitas, Disabilitas.from_str)
cattr.register_structure_hook(PenyakitDiderita, PenyakitDiderita.from_str)

cattr.register_unstructure_hook(Disabilitas, Disabilitas.todict)
cattr.register_unstructure_hook(FasilitasKesehatan, FasilitasKesehatan.todict)
cattr.register_unstructure_hook(Penghasilan, Penghasilan.todict)
cattr.register_unstructure_hook(PenyakitDiderita, PenyakitDiderita.todict)

__all__ = [
    "DataIndividu",
    "Disabilitas",
    "FasilitasKesehatan",
    "Penghasilan",
    "PenyakitDiderita",
]


def import_individu(
    sdgs: "Sdgs",
    filepath: str,
    rows: List[int],
    rt: str,
    rw: str,
):
    click.echo(f"Membuka {filepath}")
    wb = load_workbook(filepath, read_only=True)
    faileds: List[str] = list()
    failed = 0
    trying = 0
    skipped = 0
    success = 0
    for row in rows:
        if row < 4:
            raise ValueError(f"Tidak dapat memproses baris < 4")
        nik: Optional[str] = DataIndividu.get_nik(wb, row, "Individu")
        if not nik:
            raise ValueError(f"Nik kosong di baris {row}. Membatalkan operasi!")
        if not sdgs.token.token.is_valid():
            sdgs.token = sdgs.token_refresh(sdgs.token)
        if not sdgs.validateNik(nik):
            click.echo(f"NIK {nik} sudah diinput, melewati baris {row} ...")
            skipped += 1
            continue
        click.echo(f"Mempersiapkan data nik {nik}")
        data = DataIndividu.make(wb, row)
        try:
            individu: DataIndividu = cattr.structure(data, DataIndividu)
        except ValueError as e:
            click.echo(f"Baris {row} dilewati karena : {e}")
            skipped += 1
            continue
        except Exception as e:
            click.echo(f"Error ketika membuat DataKeluarga baris {row} : {e}")
            failed += 1
            continue
        if not sdgs.token.token.is_valid():
            sdgs.token = sdgs.token_refresh(sdgs.token)
        try:
            trying += 1
            res = sdgs.save_individu(individu=individu, rt=rt, rw=rw)
            if not res:
                click.echo(f"Gagal mengirim {individu} karena {res}")
            else:
                click.echo(f"Berhasil mengirim {individu}")
                success += 1
        except Exception as e:
            faileds.append(str(row))
            click.echo(f"Gagal menyimpan individu {nik} karena {e}")
            failed += 1
    click.echo(
        f"Berhasil : {success}\nMencoba : {trying}\n"
        f"Dilewati : {skipped}\nError : {failed}"
    )
    if faileds:
        click.echo("Baris yang gagal : " + ",".join(faileds))



import attr
import cattr
from datetime import date
from enum import Enum
from openpyxl import Workbook
from openpyxl.worksheet.worksheet import Worksheet
from typing import Any, Dict, List, Optional

from sdgs_tools.utils import parse_range
from .enums import (
    Agama,
    AksesInternet,
    JenisKelamin,
    KecepatanInternet,
    KeterbukaanDesa,
    KondisiPekerjaan,
    PekerjaanUtama,
    PelayananDesa,
    Pendidikan,
    StatusPernikahan,
    Warganegara,
    YaTidak,
)

from . import (
    Disabilitas,
    FasilitasKesehatan,
    Penghasilan,
    PenyakitDiderita,
)
from .mapping import MAPPING, MAPPING_COLS


@attr.dataclass(kw_only=True)
class DataIndividu:
    no_kk: str
    nik: str
    nama: str
    jenis_kelamin: JenisKelamin
    tempat_lahir: str
    tanggal_lahir: date
    usia: str
    status_pernikahan: StatusPernikahan = StatusPernikahan.BELUM_KAWIN
    usia_menikah: str
    agama: Agama
    suku_bangsa: str
    warga_negara: Warganegara = Warganegara.WNI
    nomor_hp: str
    nomor_whatsapp: Optional[str] = None
    alamat_email: Optional[str] = None
    alamat_facebook: Optional[str] = None
    alamat_twitter: Optional[str] = None
    alamat_instagram: Optional[str] = None
    aktif_internet: YaTidak
    akses_melalui: Optional[AksesInternet] = None
    kecepatan_internet: KecepatanInternet
    # Pekerjaan
    kondisi_pekerjaan: KondisiPekerjaan
    pekerjaan_utama: Optional[PekerjaanUtama] = None
    pekerjaan_utama_comment: Optional[str] = None
    jamsos_ketenagakerjaan: Optional[YaTidak] = None
    penghasilan: List[Penghasilan] = attr.field(factory=list)
    pekerjaan_penghasilan: Optional[str] = None
    # Kesehatan
    penyakit_diderita: PenyakitDiderita
    fasilitas_kesehatan: FasilitasKesehatan
    jamsos_kesehatan: YaTidak
    disabilitas: Disabilitas
    setahun_melahirkan: Optional[YaTidak] = None
    mendapat_asi: Optional[YaTidak] = None
    # Pendidikan
    pendidikan_tertinggi: Pendidikan
    tahun_pendidikan: Optional[str] = None
    pendidikan_diikuti: Optional[Pendidikan] = None
    pelatihan_diikuti: Optional[str] = None
    bahasa_permukiman: str
    bahasa_formal: str
    kerja_bakti: str = "0"
    siskamling: str = "0"
    pesta_rakyat: str = "0"
    menolong_kematian: str = "0"
    menolong_sakit: str = "0"
    menolong_kecelakaan: str = "0"
    memperoleh_pelayanan_desa: YaTidak
    pelayanan_desa: Optional[PelayananDesa] = None
    saran_desa: YaTidak
    keterbukaan_desa: Optional[KeterbukaanDesa] = None
    terjadi_bencana: YaTidak
    terdampak_bencana: Optional[YaTidak] = None

    def __str__(self):
        return f"{self.nama} ({self.nik})"

    def __attrs_post_init(self):
        if self.status_pernikahan != StatusPernikahan.BELUM_KAWIN:
            raise ValueError("Mohon diisi status pernikahan")
        if self.kondisi_pekerjaan == KondisiPekerjaan.BEKERJA:
            if not self.pekerjaan_utama:
                raise ValueError(
                    "Jika bekerja, mohon diisi pekerjaan utama dan penghasilan"
                )
            if (
                self.pekerjaan_utama == PekerjaanUtama.LAINNYA
                and not self.pekerjaan_utama_comment
            ):
                raise ValueError(
                    "Jika pekerjaan utama Lainnya, mohon diisi Nama Pekerjaan Lainnya"
                )
            if not self.penghasilan:
                raise ValueError("Jika bekerja mohon diisi Baris Penghasilan")
            if not self.pekerjaan_penghasilan:
                raise ValueError("Jika bekerja mohon diisi Penghasilan Setahun")
        else:
            self.pekerjaan_utama = None
            self.pekerjaan_utama_comment = None
        if self.terjadi_bencana and self.terdampak_bencana is None:
            raise ValueError(
                "Jika Terjadi bencana mohon diisi terdampak bencana atau tidak"
            )

    def make_data(self, desa: str, rt: str, rw: str):
        raw_data: Dict[str, Any] = cattr.unstructure(self)
        clean_data: Dict[str, Any] = dict()
        for key, properti in MAPPING.items():
            value = raw_data.get(properti)
            if value in (None, "None"):
                # TODO Improve empty value detection
                continue
            clean_data[key] = value
        clean_data.update(
            {
                "desa": desa,
                "kecamatan": desa[0:7],
                "kota": desa[0:4],
                "provinsi": desa[0:2],
                "rt": rt,
                "rw": rw,
            }
        )
        return clean_data

    @staticmethod
    def make(wb: Workbook, row: int):
        data: Dict[str, Any] = dict()
        individu: Worksheet = wb["Individu"]
        for name, col in MAPPING_COLS.items():
            data[name] = individu[f"{col}{row}"].value
        # Penghasilan
        penghasilans = individu[f"Z{row}"].value
        if penghasilans:
            data["penghasilan"] = Penghasilan.make(
                ws=wb["Penghasilan"],
                rows=parse_range(penghasilans),
            )
        else:
            data["penghasilan"] = [Penghasilan.default()]
        # Fasilitas kesehatan
        data["fasilitas_kesehatan"] = FasilitasKesehatan.make(
            ws=individu,
            row=row,
        )
        data["disabilitas"] = individu[f"AT{row}"].value
        return data

    @staticmethod
    def get_nik(
        wb: Workbook,
        row: int,
        individu_ws: str = "Individu",
    ):
        individu = wb[individu_ws]
        return individu[f"B{row}"].value

    def save(
        self,
        wb: Workbook,
        row_individu: int,
        row_penghasilan: int,
        individu_ws: str = "Individu",
        penghasilan_ws: str = "Penghasilan",
    ) -> int:
        individu: Worksheet = wb[individu_ws]
        MAPPING = dict(MAPPING_COLS)
        MAPPING.pop("penyakit_diderita")
        for name, col in MAPPING.items():
            value = getattr(self, name)
            if value is None:
                continue
            if isinstance(value, Enum):
                value = str(value)
            individu[f"{col}{row_individu}"] = value
        if self.penghasilan:
            end = row_penghasilan + len(self.penghasilan) - 1
            if row_penghasilan == end:
                individu[f"Z{row_individu}"] = row_individu
            else:
                baris_penghasilan = f"{row_penghasilan}-{end}"
                individu[f"Z{row_individu}"] = baris_penghasilan
        for penghasilan in self.penghasilan:
            if not penghasilan:
                continue
            penghasilan.save(wb[penghasilan_ws], row_penghasilan)
            row_penghasilan += 1
        self.fasilitas_kesehatan.save(individu, row_individu)
        self.penyakit_diderita.save(individu, row_individu)
        individu[f"AT{row_individu}"] = str(self.disabilitas)
        return row_penghasilan



import attr
import cattr
from openpyxl.worksheet.worksheet import Worksheet
from typing import Dict

from . import Akses

MAPPING = {
    "K.P421_1PAUD": "paud",
    "K.P421_2TK": "tk",
    "K.P421_3SD": "sd",
    "K.P421_4SMP": "smp",
    "K.P421_5SMA": "sma",
    "K.P421_6PT": "pt",
    "K.P421_7Pesantren": "pesantren",
    "K.P421_8Seminari": "seminari",
    "K.P421_9lainnya": "lainnya",
}

MAPPING_VALUE = {
    "paud": ("AL", "AM", "AN"),
    "tk": ("AO", "AP", "AQ"),
    "sd": ("AR", "AS", "AT"),
    "smp": ("AU", "AV", "AW"),
    "sma": ("AX", "AY", "AZ"),
    "pt": ("BA", "BB", "BC"),
    "pesantren": ("BD", "BE", "BF"),
    "seminari": ("BG", "BH", "BI"),
    "lainnya": ("BJ", "BK", "BL"),
}


@attr.dataclass
class AksesPendidikan:
    paud: Akses = attr.field(default=attr.Factory(Akses))
    tk: Akses = attr.field(default=attr.Factory(Akses))
    sd: Akses = attr.field(default=attr.Factory(Akses))
    smp: Akses = attr.field(default=attr.Factory(Akses))
    sma: Akses = attr.field(default=attr.Factory(Akses))
    pt: Akses = attr.field(default=attr.Factory(Akses))
    pesantren: Akses = attr.field(default=attr.Factory(Akses))
    seminari: Akses = attr.field(default=attr.Factory(Akses))
    lainnya: Akses = attr.field(default=attr.Factory(Akses))

    def todict(self) -> Dict[str, Dict[str, str]]:
        data: Dict[str, Dict[str, str]] = dict()
        for key, name in MAPPING.items():
            value: Akses = getattr(self, name)
            data[key] = cattr.unstructure(value)
        return data

    @staticmethod
    def make(ws: Worksheet, row: int) -> Dict[str, Dict[str, str]]:
        data: Dict[str, Dict[str, str]] = dict()
        for name, args in MAPPING_VALUE.items():
            data[name] = Akses.from_cols(ws, row, *args)
        return data

    def save(self, ws: Worksheet, row: int):
        for name, args in MAPPING_VALUE.items():
            akses: Akses = getattr(self, name)
            akses.save(ws, row, *args)



from enum import Enum
from sdgs_tools.dashboard.utils import make_str_to_enum, make_dunder_str

MAPPING = {
    "Tidak Sekolah": "1",
    "SD": "2",
    "SMP": "3",
    "SMA": "4",
    "Diploma": "5",
    "S1": "6",
    "S2": "7",
    "S3": "8",
    "Pesantren / Seminari": "9",
    "Lainnya": "10",
}


class Pendidikan(Enum):
    TIDAK_SEKOLAH = "1"
    SD = "2"
    SMP = "3"
    SMA = "4"
    DIPLOMA = "5"
    S1 = "6"
    S2 = "7"
    S3 = "8"
    PESANTREN_SEMINARI = "9"
    LAINNYA = "10"

    from_str = classmethod(make_str_to_enum(MAPPING, "1"))
    __str__ = make_dunder_str(MAPPING)



import attr
import cattr
from openpyxl.worksheet.worksheet import Worksheet
from typing import Dict

from . import SarprasTransport

MAPPING = {
    "K.P424_1pekerjaan": "pekerjaan",
    "K.P424_2pertanian": "pertanian",
    "K.P424_3sekolah": "sekolah",
    "K.P424_4berobat": "berobat",
    "K.P424_5ibadah": "ibadah",
    "K.P424_6rekreasi": "rekreasi",
}

MAPPING_VALUE = {
    "pekerjaan": ("DF", "DG", "DH", "DI", "DJ"),
    "pertanian": ("DK", "DL", "DM", "DN", "DO"),
    "sekolah": ("DP", "DQ", "DR", "DS", "DT"),
    "berobat": ("DU", "DV", "DW", "DX", "DY"),
    "ibadah": ("DZ", "EA", "EB", "EC", "ED"),
    "rekreasi": ("EE", "EF", "EG", "EH", "EI"),
}


@attr.dataclass
class AksesSarprasTransport:
    pekerjaan: SarprasTransport = attr.field(default=attr.Factory(SarprasTransport))
    pertanian: SarprasTransport = attr.field(default=attr.Factory(SarprasTransport))
    sekolah: SarprasTransport = attr.field(default=attr.Factory(SarprasTransport))
    berobat: SarprasTransport = attr.field(default=attr.Factory(SarprasTransport))
    ibadah: SarprasTransport = attr.field(default=attr.Factory(SarprasTransport))
    rekreasi: SarprasTransport = attr.field(default=attr.Factory(SarprasTransport))

    def todict(self) -> Dict[str, Dict[str, str]]:
        data: Dict[str, Dict[str, str]] = dict()
        for key, name in MAPPING.items():
            value: SarprasTransport = getattr(self, name)
            data[key] = cattr.unstructure(value)
        return data

    @staticmethod
    def make(ws: Worksheet, row: int) -> Dict[str, Dict[str, str]]:
        data: Dict[str, Dict[str, str]] = dict()
        for name, args in MAPPING_VALUE.items():
            data[name] = SarprasTransport.from_cols(ws, row, *args)
        return data

    def save(self, ws: Worksheet, row: int):
        for name, args in MAPPING_VALUE.items():
            sarpras: SarprasTransport = getattr(self, name)
            sarpras.save(ws, row, *args)



from enum import Enum
from sdgs_tools.dashboard.utils import make_str_to_enum, make_dunder_str

MAPPING = {
    "Tangki/instalasi pengelolaan limbah": "1",
    "Sawah/kolam/sungai/drainase/laut": "2",
    "Lubang di tanah": "3",
    "Lainnya": "other",
}


class PembuanganLimbahCair(Enum):
    PENGOLAHAN_LIMBAH = "1"
    SALURAN_AIR_TERBUKA = "2"
    LUBANG_TANAH = "3"
    LAINNYA = "other"

    from_str = classmethod(make_str_to_enum(MAPPING, "other"))
    __str__ = make_dunder_str(MAPPING)



from enum import Enum
from sdgs_tools.dashboard.utils import make_str_to_enum, make_dunder_str

MAPPING = {
    "Bersekolah": 1,
    "Ibu Rumah Tangga": 2,
    "Tidak Bekerja": 3,
    "Sedang Mencari Pekerjaan": 4,
    "Bekerja": 5,
}


class KondisiPekerjaan(Enum):
    BERSEKOLAH = 1
    IBU_RUMAH_TANGGA = 2
    TIDAK_BEKERJA = 3
    SEDANG_MENCARI_PEKERJAAN = 4
    BEKERJA = 5

    from_str = classmethod(make_str_to_enum(MAPPING, 3))
    __str__ = make_dunder_str(MAPPING)



from enum import Enum
from sdgs_tools.dashboard.utils import make_str_to_enum, make_dunder_str

MAPPING = {
    "Ledeng/perpipaan berbayar/air isi ulang/kemasan": "1",
    "Perpipaan": "2",
    "Mata air/ sumur": "3",
    "Sungai, danau, embung": "4",
    "Tadah air hujan ": "5",
    "Lainnya": "other",
    "Ledeng/perpipaan berbayar/air isi ulang/kemasa": "1",
}


class SumberAirMandi(Enum):
    PERPIPAAN_BERBAYAR = "1"
    PERPIPAAN = "2"
    MATA_AIR_SUMUR = "3"
    SUNGAI_DANAU_EMBUNG = "4"
    TADAH_AIR_HUJAN = "5"
    LAINNYA = "other"

    from_str = classmethod(make_str_to_enum(MAPPING, "1"))
    __str__ = make_dunder_str(MAPPING)



import attr
from enum import Enum
from openpyxl.worksheet.worksheet import Worksheet
from typing import Any, Dict, List, Optional

from .enums import Diekspor
from .enums import SumberPenghasilan


MAPPING_COLS = {
    # Sheet Penghasilan
    "sumber_penghasilan": "A",
    "penghasilan_comment": "B",
    "penghasilan_jumlah": "C",
    "penghasilan_setahun": "D",
    "penghasilan_diekspor": "E",
}


@attr.dataclass
class Penghasilan:
    penghasilan_diekspor: Optional[Diekspor] = None
    penghasilan_jumlah: Optional[str] = None
    penghasilan_setahun: Optional[str] = None
    sumber_penghasilan: Optional[SumberPenghasilan] = None
    penghasilan_comment: Optional[str] = None

    def __bool__(self):
        return self.sumber_penghasilan is not None

    def todict(self) -> Dict[str, Optional[str]]:
        data: Dict[str, Optional[str]] = {
            "diekspor": None,
            "jumlah": None,
            "penghasilan": None,
            "sumber_penghasilan": None,
        }
        if self.penghasilan_diekspor is not None:
            data["diekspor"] = str(self.penghasilan_diekspor.value)
        if self.penghasilan_jumlah is not None:
            data["jumlah"] = str(self.penghasilan_jumlah)
        if self.penghasilan_setahun is not None:
            data["penghasilan"] = str(self.penghasilan_setahun)
        if self.sumber_penghasilan is not None:
            data["sumber_penghasilan"] = str(self.sumber_penghasilan.value)
        if self.sumber_penghasilan is None:
            pass
        elif self.sumber_penghasilan.value == "other":
            if not self.penghasilan_comment:
                raise ValueError(
                    "comment harus diisi jika sumber_penghasilan = Lainnya"
                )
            data["sumber_penghasilan-Comment"] = self.penghasilan_comment
        return data

    @classmethod
    def default(
        cls,
        penghasilan_diekspor: Diekspor = None,
        penghasilan_jumlah: str = None,
        penghasilan_setahun: str = None,
        sumber_penghasilan: SumberPenghasilan = None,
        penghasilan_comment: str = None,
    ):
        return attr.asdict(
            cls(
                penghasilan_diekspor=penghasilan_diekspor,
                penghasilan_jumlah=penghasilan_jumlah,
                penghasilan_setahun=penghasilan_setahun,
                sumber_penghasilan=sumber_penghasilan,
                penghasilan_comment=penghasilan_comment,
            )
        )

    @staticmethod
    def make(ws: Worksheet, rows: List[int]):
        results: List[Any] = list()
        for row in rows:
            data: Dict[str, Any] = dict()
            for name, col in MAPPING_COLS.items():
                data[name] = ws[f"{col}{row}"].value
            results.append(data)
        return results

    def save(self, ws: Worksheet, row: int):
        for name, col in MAPPING_COLS.items():
            value = getattr(self, name)
            if value is None:
                continue
            elif isinstance(value, Enum):
                value = str(value)
            ws[f"{col}{row}"] = value



import tkinter as tk


class TextForm:
    def __init__(
        self,
        master,
        text: str,
        row: int,
        value: str = None,
        name: str = None,
        start_col: int = 0,
        password: bool = False,
    ):
        super().__init__()
        self.value = tk.StringVar(master, value, name)
        self.label = tk.Label(master, text=text)
        self.label.grid(row=row, column=start_col)
        if password:
            self.entry = tk.Entry(master, textvariable=self.value, show="*")
        else:
            self.entry = tk.Entry(master, textvariable=self.value)
        self.entry.grid(row=row, column=start_col + 1)

    def __str__(self):
        return self.value.get()

    def get(self):
        return self.value.get()



import tkinter as tk
from tkinter.messagebox import showinfo

from sdgs_tools import __version__
from . import ContentFrame
from . import SidebarFrame

from . import AplikasiTab
from . import DashboardTab


class MainApp(tk.Tk):
    def __init__(self):
        super().__init__(screenName="SDGS Tools")
        self.title(f"SDGS Tools v{__version__}")
        self.sidebar = SidebarFrame(self)
        self.content = ContentFrame(self)

        self.aplikasi_tab = AplikasiTab(self.content)
        self.dashboard_tab = DashboardTab(self.content)
        self.sidebar.add_button("Aplikasi SDGS", self.aplikasi_tab.tkraise)
        self.sidebar.add_button("Dashboard SDGS", self.dashboard_tab.tkraise)

        self.sidebar.add_button("Tentang", self.about)

    def __call__(self):
        self.mainloop()

    def about(self):
        showinfo(
            "Tentang",
            f"SDGS-Tools versi{__version__} dibuat oleh Habib Rohman\n"
            "Donasi : https://saweria.co/hexatester\n"
            "Error? chat : https://t.me/hexatester\n"
            "Aplikasi ini sama sekali tidak berafiliasi dengan, diizinkan, dipelihara, disponsori atau didukung oleh Kemendesa atau afiliasi atau anak organisasinya.\n"
            "Ini adalah perangkat lunak yang independen dan tidak resmi. Gunakan dengan risiko Anda sendiri.",
        )



from typing import Any, Dict
from uiautomator2 import Device
from uiautomator2.exceptions import UiObjectNotFoundError

from sdgs_tools.aplikasi_sdgs.utils import d_get_text, menu_to

PENDIDIKAN_SUBTITUTE = {
    "Tidak Sekolah": "Tidak Sekolah",
    "SD dan sederajat": "SD",
    "SMP dan sederajat": "SMP",
    "SMA dan sederajat": "SMA",
    "Diploma 1-3": "Diploma",
    "S1 dan sederajat": "S1",
    "S2 dan sederajat": "S2",
    "S3 dan sederajat": "S3",
    "Pesanteren, seminari, wihara dan sejenisnya": "Pesantren / Seminari",
    "Lainnya": "Lainnya",
}

# resourceId
PENDIDIKAN_COL = {
    "pendidikan_tertinggi": "com.kemendes.survey:id/cbPendidikanTertinggi",
    "tahun_pendidikan": "com.kemendes.survey:id/txtTahunPendidikanDasar",
    # pendidikan_diikuti?
    # pelatihan_diikuti?
    "bahasa_permukiman": "com.kemendes.survey:id/txtBahasa",
    "bahasa_formal": "com.kemendes.survey:id/txtBahasaLembaga",
    "kerja_bakti": "com.kemendes.survey:id/txtKerjaBakti",
    "siskamling": "com.kemendes.survey:id/txtSiskamling",
    "pesta_rakyat": "com.kemendes.survey:id/txtPestaRakyat",
    "menolong_kematian": "com.kemendes.survey:id/txtMenolongKematian",
    "menolong_sakit": "com.kemendes.survey:id/txtMenolongSakit",
    "menolong_kecelakaan": "com.kemendes.survey:id/txtMenolongKecelakaan",
    "memperoleh_pelayanan_desa": "com.kemendes.survey:id/cbPerolehPelayananDesa",
    "pelayanan_desa": "com.kemendes.survey:id/cbPelayananDesa",
    "saran_desa": "com.kemendes.survey:id/cbMasukan",
    "keterbukaan_desa": "com.kemendes.survey:id/cbKeterbukaan",
    "terjadi_bencana": "com.kemendes.survey:id/cbBencana",
    "terdampak_bencana": "com.kemendes.survey:id/cbTerkenaBencana",
}


def get_data_pendidikan(d: Device) -> Dict[str, Any]:
    menu_to(d, "PENDIDIKAN")
    data: Dict[str, Any] = dict()
    d(text="PENDIDIKAN").click()
    for name, resourceId in PENDIDIKAN_COL.items():
        try:
            data[name] = d_get_text(d, resourceId)
        except UiObjectNotFoundError:
            pass
    d(className="android.widget.ScrollView").fling.vert.backward()
    data["pendidikan_tertinggi"] = PENDIDIKAN_SUBTITUTE[data["pendidikan_tertinggi"]]
    return data



import click

from sdgs_tools.utils import parse_range
from sdgs_tools.dashboard.sdgs import Sdgs
from sdgs_tools.dashboard.import_individu import import_individu as _import_individu


@click.command("import-individu")
@click.option("--username", required=True, help="Username dashboard-sdgs")
@click.password_option("--password", required=True, help="Password dashboard-sdgs")
@click.option("--rt", required=True, type=str)
@click.option("--rw", required=True, type=str)
@click.option(
    "--baris",
    required=True,
    type=str,
    help="Baris yang akan diinputkan dari excel",
)
@click.argument("nama-file", type=click.Path(exists=True), required=True)
def import_individu(
    username: str,
    password: str,
    rt: str,
    rw: str,
    baris: str,
    nama_file: str,
):
    try:
        _import_individu(
            sdgs=Sdgs(username, password),
            filepath=nama_file,
            rows=parse_range(baris),
            rt=rt,
            rw=rw,
        )
    except FileNotFoundError:
        click.echo("Gagal mengeksport data karena file template tidak ditemukan")
    except Exception as e:
        click.echo(f"Terjadi error : {e}")



from enum import Enum
from sdgs_tools.dashboard.utils import make_str_to_enum, make_dunder_str

MAPPING = {
    "Belum Kawin": "1",
    "Kawin": "2",
    "Cerai Hidup": "3",
    "Cerai Mati": "4",
}


class StatusPernikahan(Enum):
    BELUM_KAWIN = "1"
    KAWIN = "2"
    CERAI_HIDUP = "3"
    CERAI_MATI = "4"

    from_str = classmethod(make_str_to_enum(MAPPING, "1"))
    __str__ = make_dunder_str(MAPPING, "Belum Kawin")



import requests
import tkinter as tk
import webbrowser
from tkinter.filedialog import asksaveasfile
from tkinter.messagebox import showinfo, showwarning

from .import_individu import ImportIndividuWindow
from .import_keluarga import ImportKeluargaWindow


class DashboardTab(tk.Frame):
    TEMPLATE_INDIVIDU = "https://hexatester.github.io/sdgs-tools/Template%20Individu%20Dashboard%20SDGS%20Kemendesa.xlsm"
    TEMPLATE_KELUARGA = "https://hexatester.github.io/sdgs-tools/Template%20Keluarga%20Dashboard%20SDGS%20Kemendesa.xlsm"

    def __init__(self, master: tk.Frame, **kw):
        super().__init__(master, bg="#CCC", width=500, height=500, **kw)
        # self.pack(expand=True, fill="both", side="right")
        self.grid(row=0, column=0, sticky="nsew")
        self.label_info = tk.Label(
            self,
            text="Fitur ini untuk meng import data ke dashboard-sdgs",
        )
        self.label_info.grid(row=0, column=0, columnspan=2)
        # Individu
        self.template_individu_button = tk.Button(
            self,
            text="Buat Template Individu",
            command=self.download_template_individu,
        )
        self.template_individu_button.grid(row=1, column=0)
        self.import_individu_button = tk.Button(
            self,
            text="Import Individu",
            command=ImportIndividuWindow,
        )
        self.import_individu_button.grid(row=1, column=1)
        # Keluarga
        self.template_keluarga_button = tk.Button(
            self,
            text="Buat Template Keluarga",
            command=self.download_template_keluarga,
        )
        self.template_keluarga_button.grid(row=2, column=0)
        self.import_keluarga_button = tk.Button(
            self,
            text="Import Keluarga",
            command=ImportKeluargaWindow,
        )
        self.import_keluarga_button.grid(row=2, column=1)
        self.donate_label = tk.Label(
            self, text="Donasi : https://saweria.co/hexatester"
        )
        self.donate_label.grid(row=3, column=0, columnspan=3)

    def download_template_individu(self):
        files = [
            ("Excel 2010+", "*.xlsx"),
        ]
        filepath = asksaveasfile(filetypes=files, defaultextension=files)
        if not filepath or not filepath.name:
            showwarning(
                "Gagal",
                "Mohon tentukan nama dan lokasi file yang akan disimpan",
            )
        res = requests.get(self.TEMPLATE_INDIVIDU)
        with open(filepath.name, "wb") as f:
            f.write(res.content)
        showinfo("Sukses", f"Berhasil menyimpan template di {filepath.name}")

    def download_template_keluarga(self):
        files = [
            ("Excel 2010+", "*.xlsx"),
        ]
        filepath = asksaveasfile(filetypes=files, defaultextension=files)
        if not filepath or not filepath.name:
            showwarning(
                "Gagal",
                "Mohon tentukan nama dan lokasi file yang akan disimpan",
            )
        res = requests.get(self.TEMPLATE_KELUARGA)
        with open(filepath.name, "wb") as f:
            f.write(res.content)
        showinfo("Sukses", f"Berhasil menyimpan template di {filepath.name}")



from ..response import SdgsResponse
from ..base import BaseSdgs

from . import AuthToken


class BaseAuth(BaseSdgs):
    def login(
        self, username: str, password: str, userType: str = "enumerator"
    ) -> AuthToken:
        res = self.session.get("https://dashboard-sdgs.kemendesa.go.id/")
        # AuthToken
        if not res.ok:
            raise Exception("Server tidak merespon, mohon coba beberapa saat lagi")
        url = self.url_api("auth/token")
        data = {"username": username, "password": password}
        res = self.session.post(url, data)
        if not res.ok:
            raise Exception("Server tidak merespon, mohon coba beberapa saat lagi")
        res_data = SdgsResponse.from_str(res.text, AuthToken)
        if res_data.status == 500:
            raise Exception(res_data.message)
        self.session.headers.update(res_data.data.token.token_header)
        if res_data.data.user.userType.nama != userType:
            raise ValueError(f"Mohon login menggunakan user bertipe {userType}")
        return res_data.data

    def token_refresh(self, token: AuthToken) -> AuthToken:
        url = self.url_api("auth/token/refresh")
        data = {
            "refreshToken": token.token.refresh_token,
            "username": token.user.username,
        }
        res = self.session.post(url, data)
        if not res.ok:
            raise Exception("Server tidak merespon")
        res_data = SdgsResponse.from_str(res.text, AuthToken)
        self.session.headers.update(res_data.data.token.token_header)
        return res_data.data



import attr
import cattr
from typing import Any, Dict, Generic, Optional, Type, TypeVar

try:
    import ujson as json
except ImportError:
    import json  # type: ignore[no-redef]


T = TypeVar("T")


@attr.dataclass
class SdgsResponse(Generic[T]):
    status: int
    message: str
    data: T

    def __bool__(self):
        return self.message == "SUCCESS"

    @classmethod
    def from_str(
        cls,
        data_str: str,
        cl: Type[T],
        ignore: bool = False,
    ) -> "SdgsResponse":
        data: Dict[str, Any] = json.loads(data_str)
        if data["status"] == 500 and not ignore:
            raise Exception(data["message"])
        return cls(
            status=data["status"],
            message=data["message"],
            data=cattr.structure(data["data"], cl),
        )



import attr
import cattr
from openpyxl.worksheet.worksheet import Worksheet
from typing import Dict

from . import Akses

MAPPING = {
    "K.P423_1spesialis": "spesialis",
    "K.P423_2dokter": "dokter",
    "K.P423_3Bidan": "bidan",
    "K.P423_4nakes": "nakes",
    "K.P423_5Dukun": "dukun",
}

MAPPING_VALUE = {
    "spesialis": ("CQ", "CR", "CS"),
    "dokter": ("CT", "CU", "CV"),
    "bidan": ("CW", "CX", "CY"),
    "nakes": ("CZ", "DA", "DB"),
    "dukun": ("DC", "DD", "DE"),
}


@attr.dataclass
class AksesTenagaKesehatan:
    spesialis: Akses = attr.field(default=attr.Factory(Akses))
    dokter: Akses = attr.field(default=attr.Factory(Akses))
    bidan: Akses = attr.field(default=attr.Factory(Akses))
    nakes: Akses = attr.field(default=attr.Factory(Akses))
    dukun: Akses = attr.field(default=attr.Factory(Akses))

    def todict(self) -> Dict[str, Dict[str, str]]:
        data: Dict[str, Dict[str, str]] = dict()
        for key, name in MAPPING.items():
            value: Akses = getattr(self, name)
            data[key] = cattr.unstructure(value)
        return data

    @staticmethod
    def make(ws: Worksheet, row: int) -> Dict[str, Dict[str, str]]:
        data: Dict[str, Dict[str, str]] = dict()
        for name, args in MAPPING_VALUE.items():
            data[name] = Akses.from_cols(ws, row, *args)
        return data

    def save(self, ws: Worksheet, row: int):
        for name, args in MAPPING_VALUE.items():
            akses: Akses = getattr(self, name)
            akses.save(ws, row, *args)



import attr
from openpyxl.worksheet.worksheet import Worksheet
from typing import Dict, List, Optional

from .enums import YaTidak
from .mapping import MAPPING_COLS


MAPPING = {
    "Mutaber Diare": "mutaber_diare",
    "Demam Berdarah": "demam_berdarah",
    "Campak": "campak",
    "Malaria": "malaria",
    "Flu Burung Sars": "flu_burung_sars",
    "Covid19": "covid19",
    "Hepatitis B": "hepatitis_b",
    "Hepatitis E": "hepatitis_e",
    "Difteri": "difteri",
    "Chikungunya": "chikungunya",
    "Leptospirosis": "leptospirosis",
    "Kolera": "kolera",
    "Gizi Buruk": "gizi_buruk",
    "Jantung": "jantung",
    "TBC Paru-Paru": "tbc_paru_paru",
    "Kanker": "kanker",
    "Diabetes": "diabetes",
    "Lumpuh": "lumpuh",
    "Lainnya": "lainnya",
}


@attr.dataclass
class PenyakitDiderita:
    # P404
    mutaber_diare: YaTidak = YaTidak.TIDAK
    demam_berdarah: YaTidak = YaTidak.TIDAK
    campak: YaTidak = YaTidak.TIDAK
    malaria: YaTidak = YaTidak.TIDAK
    flu_burung_sars: YaTidak = YaTidak.TIDAK
    covid19: YaTidak = YaTidak.TIDAK
    hepatitis_b: YaTidak = YaTidak.TIDAK
    hepatitis_e: YaTidak = YaTidak.TIDAK
    difteri: YaTidak = YaTidak.TIDAK
    chikungunya: YaTidak = YaTidak.TIDAK
    leptospirosis: YaTidak = YaTidak.TIDAK
    kolera: YaTidak = YaTidak.TIDAK
    gizi_buruk: YaTidak = YaTidak.TIDAK
    jantung: YaTidak = YaTidak.TIDAK
    tbc_paru_paru: YaTidak = YaTidak.TIDAK
    kanker: YaTidak = YaTidak.TIDAK
    diabetes: YaTidak = YaTidak.TIDAK
    lumpuh: YaTidak = YaTidak.TIDAK
    lainnya: YaTidak = YaTidak.TIDAK

    def todict(self) -> Dict[str, str]:
        return {
            "1": "1" if self.mutaber_diare else "2",
            "2": "1" if self.demam_berdarah else "2",
            "3": "1" if self.campak else "2",
            "4": "1" if self.malaria else "2",
            "5": "1" if self.flu_burung_sars else "2",
            "6": "1" if self.covid19 else "2",
            "7": "1" if self.hepatitis_b else "2",
            "8": "1" if self.hepatitis_e else "2",
            "9": "1" if self.difteri else "2",
            "10": "1" if self.chikungunya else "2",
            "11": "1" if self.leptospirosis else "2",
            "12": "1" if self.kolera else "2",
            "13": "1" if self.gizi_buruk else "2",
            "14": "1" if self.jantung else "2",
            "15": "1" if self.tbc_paru_paru else "2",
            "16": "1" if self.kanker else "2",
            "17": "1" if self.diabetes else "2",
            "18": "1" if self.lumpuh else "2",
            "19": "1" if self.lainnya else "2",
        }

    @classmethod
    def from_str(cls, value: Optional[str], t=None):
        if not value:
            return cls()
        elif isinstance(value, dict):
            return cls.from_dict(value)
        data: Dict[str, YaTidak] = dict()
        for key, name in MAPPING.items():
            if value in key:
                data[name] = YaTidak.YA
            else:
                data[name] = YaTidak.TIDAK
        return cls(**data)

    @classmethod
    def from_dict(cls, val: Dict[str, str]):
        data: Dict[str, YaTidak] = dict()
        for name, value in val.items():
            if value == "Ya":
                data[name] = YaTidak.YA
            else:
                data[name] = YaTidak.TIDAK
        return cls(**data)

    def save(self, ws: Worksheet, row: int):
        results: List[str] = list()
        for key, name in MAPPING.items():
            value = getattr(self, name)
            if value == "Ya":
                results.append(key)
        if not results:
            return
        col: str = MAPPING_COLS["penyakit_diderita"]
        ws[f"{col}{row}"] = ", ".join(results)



import cattr

from .atap import Atap
from .dinding import Dinding
from .energi_memasak import EnergiMemasak
from .fasilitas_bab import FasilitasBab
from .fasilitas_mck import FasilitasMck
from .jendela import Jendela
from .jenis_transportasi import JenisTransportasi
from .lantai import Lantai
from .pembuangan_limbah_cair import PembuanganLimbahCair
from .pembuangan_sampah import PembuanganSampah
from .penerangan import Penerangan
from .status_lahan import StatusLahan
from .sumber_air_mandi import SumberAirMandi
from .sumber_air_minum import SumberAirMinum
from .sumber_kayu_bakar import SumberKayuBakar
from .tempat_tinggal import TempatTinggal
from .ya_tidak import YaTidak

cattr.register_structure_hook(Atap, Atap.from_str)  # type: ignore
cattr.register_structure_hook(Dinding, Dinding.from_str)  # type: ignore
cattr.register_structure_hook(EnergiMemasak, EnergiMemasak.from_str)  # type: ignore
cattr.register_structure_hook(FasilitasBab, FasilitasBab.from_str)  # type: ignore
cattr.register_structure_hook(FasilitasMck, FasilitasMck.from_str)  # type: ignore
cattr.register_structure_hook(Jendela, Jendela.from_str)  # type: ignore
cattr.register_structure_hook(JenisTransportasi, JenisTransportasi.from_str)  # type: ignore
cattr.register_structure_hook(Lantai, Lantai.from_str)  # type: ignore
cattr.register_structure_hook(PembuanganLimbahCair, PembuanganLimbahCair.from_str)  # type: ignore
cattr.register_structure_hook(PembuanganSampah, PembuanganSampah.from_str)  # type: ignore
cattr.register_structure_hook(Penerangan, Penerangan.from_str)  # type: ignore
cattr.register_structure_hook(StatusLahan, StatusLahan.from_str)  # type: ignore
cattr.register_structure_hook(SumberAirMandi, SumberAirMandi.from_str)  # type: ignore
cattr.register_structure_hook(SumberAirMinum, SumberAirMinum.from_str)  # type: ignore
cattr.register_structure_hook(SumberKayuBakar, SumberKayuBakar.from_str)  # type: ignore
cattr.register_structure_hook(TempatTinggal, TempatTinggal.from_str)  # type: ignore
cattr.register_structure_hook(YaTidak, YaTidak.from_str)  # type: ignore

__all__ = [
    "Atap",
    "Dinding",
    "EnergiMemasak",
    "FasilitasMck",
    "FasilitasBab",
    "Jendela",
    "JenisTransportasi",
    "Lantai",
    "PembuanganLimbahCair",
    "PembuanganSampah",
    "Penerangan",
    "StatusLahan",
    "SumberAirMandi",
    "SumberAirMinum",
    "SumberKayuBakar",
    "TempatTinggal",
    "YaTidak",
]



from typing import Any, Dict
from uiautomator2 import Device, UiObject

from sdgs_tools.aplikasi_sdgs.utils import d_get_text, menu_to, swipe_box

AKSES_SARPRAS = {
    "Lokasi pekerjaan utama": "pekerjaan",
    "Lahan pertanian yang sedang diusahakan": "pertanian",
    "Sekolah": "sekolah",
    "Berobat": "berobat",
    "Beribadah Mingguan/Bulanan/Tahunan": "ibadah",
    "Rekreasi Terdekat": "rekreasi",
}

SARPRAS_COL = {
    # "C": ("com.kemendes.survey:id/txtNama", "Tujuan : Lokasi pekerjaan utama Jenis Transportasi : Darat"),
    # "D": ("com.kemendes.survey:id/txtAlamat", "Waktu Tempuh :0.25 jam, Biaya : 5000"),
    "kemudahan": ("com.kemendes.survey:id/txtTelpon", "Kemudahan :"),
    # "H": ("com.kemendes.survey:id/txtStatus", ""),
}


def get_data_sarpras(d: Device) -> Dict[str, Dict[str, Any]]:
    menu_to(d, "AKSES SARANA PRASARANA")
    sarpras: Dict[str, Dict[str, Any]] = dict()
    box_daftar_sarpras: UiObject = d(resourceId="com.kemendes.survey:id/itemsSarana")
    survey_boxes: UiObject = box_daftar_sarpras.child(
        resourceId="com.kemendes.survey:id/box"
    )
    for survey_box in survey_boxes:
        data: Dict[str, Any] = dict()
        # Kemudahan
        kemudahan_box = survey_box.child(resourceId="com.kemendes.survey:id/txtTelpon")
        kemudahan: str = kemudahan_box.info.get("text")
        data["kemudahan"] = kemudahan.lstrip("Kemudahan :")
        # Tujuan & Jenis
        tujuan_jenis: str = survey_box.child(
            resourceId="com.kemendes.survey:id/txtNama"
        ).info.get("text")
        tujuan_jenis = tujuan_jenis.lstrip("Tujuan : ")
        tujuan_jenis = tujuan_jenis.replace(" Jenis Transportasi", "")
        tujuan, jenis = tujuan_jenis.split(":")
        nama = tujuan.strip()
        data["jenis"] = jenis.strip()
        # Waktu & Biaya
        waktu_biaya: str = survey_box.child(
            resourceId="com.kemendes.survey:id/txtAlamat"
        ).info.get("text")
        waktu_biaya = waktu_biaya.lstrip("Waktu Tempuh :")
        waktu_biaya = waktu_biaya.replace(", Biaya", "")
        waktu, biaya = waktu_biaya.split(":")
        data["waktu"] = waktu.strip().rstrip(" jam")
        data["biaya"] = biaya.strip()
        swipe_box(d, survey_box)
        sarpras[AKSES_SARPRAS[nama]] = data
    d(className="android.widget.ScrollView").fling.vert.backward()
    return sarpras



import attr
from openpyxl.worksheet.worksheet import Worksheet
from typing import Optional

from .enums import JenisTransportasi, YaTidak


@attr.dataclass
class SarprasTransport:
    jenis: Optional[JenisTransportasi] = None
    transport_umum: Optional[YaTidak] = None
    waktu: Optional[str] = None
    biaya: Optional[str] = None
    kemudahan: Optional[YaTidak] = None

    def __attrs_post_init__(self):
        if self.waktu in (None, "None"):
            self.waktu = "0"
        elif self.waktu.isdigit():
            waktu = int(self.waktu) / 60
            self.waktu = str(round(waktu, 4))
        elif self.waktu.isdecimal():
            waktu = float(self.waktu) / 60
            self.waktu = str(round(waktu, 4))
        if not self.biaya:
            self.biaya = 0

    @staticmethod
    def from_cols(
        ws: Worksheet,
        row: int,
        j: str,
        t: str,
        w: str,
        b: str,
        k: str,
    ):
        return {
            "jenis": ws[f"{j}{row}"].value,
            "transport_umum": ws[f"{t}{row}"].value,
            "waktu": ws[f"{w}{row}"].value,
            "biaya": ws[f"{b}{row}"].value,
            "kemudahan": ws[f"{k}{row}"].value,
        }

    def save(
        self,
        ws: Worksheet,
        row: int,
        j: str,
        t: str,
        w: str,
        b: str,
        k: str,
    ):
        if self.jenis is not None:
            ws[f"{j}{row}"] = self.jenis.value
        if self.transport_umum is not None:
            ws[f"{t}{row}"] = self.transport_umum.value
        if self.waktu is not None:
            ws[f"{w}{row}"] = self.waktu
        if self.biaya is not None:
            ws[f"{b}{row}"] = self.biaya
        if self.kemudahan is not None:
            ws[f"{k}{row}"] = self.kemudahan.value



import attr
from openpyxl.worksheet.worksheet import Worksheet
from typing import Any, Dict, Optional

from .enums import YaTidak


@attr.dataclass
class Akses:
    jarak: Optional[float] = None
    waktu: Optional[float] = None
    kemudahan: Optional[YaTidak] = None

    @staticmethod
    def from_cols(ws: Worksheet, row: int, j: str, w: str, k: str):
        return {
            "jarak": ws[f"{j}{row}"].value,
            "waktu": ws[f"{w}{row}"].value,
            "kemudahan": ws[f"{k}{row}"].value,
        }

    def todict(self) -> Dict[str, Any]:
        data: Dict[str, Any] = {
            "jarak": None,
            "waktu": None,
            "kemudahan": None,
        }
        if isinstance(self.jarak, float):
            if self.waktu == 0:
                data["jarak"] = "0"
            else:
                data["jarak"] = str(self.jarak)
        elif isinstance(self.jarak, int):
            data["jarak"] = str(self.jarak)
        if isinstance(self.waktu, float):
            if self.waktu == 0:
                data["waktu"] = "0"
            else:
                waktu = self.waktu / 60
                data["waktu"] = str(round(waktu, 4))
        elif isinstance(self.waktu, int):
            waktu = self.waktu / 60
            data["waktu"] = str(round(waktu, 4))
        if self.kemudahan is not None:
            data["kemudahan"] = self.kemudahan.value
        return data

    def save(self, ws: Worksheet, row: int, j: str, w: str, k: str):
        if self.jarak is not None:
            ws[f"{j}{row}"] = self.jarak
        if self.waktu is not None:
            ws[f"{w}{row}"] = self.waktu
        if self.kemudahan is not None:
            ws[f"{k}{row}"] = str(self.kemudahan)



from .content import ContentFrame
from .sidebar import SidebarFrame

from .aplikasi import AplikasiTab
from .dashboard import DashboardTab

from .main import MainApp


__all__ = [
    "AplikasiTab",
    "ContentFrame",
    "DashboardTab",
    "SidebarFrame",
    "MainApp",
]



from uiautomator2 import Device, UiObject
from typing import Any, Dict

from sdgs_tools.aplikasi_sdgs.utils import d_get_text, menu_to

MAPPING = {
    "Mutaber Diare": "mutaber_diare",
    "Demam Berdarah": "demam_berdarah",
    "Campak": "campak",
    "Malaria": "malaria",
    "Flu Burung Sars": "flu_burung_sars",
    "Covid19": "covid19",
    "Hepatitis B": "hepatitis_b",
    "Hepatitis E": "hepatitis_e",
    "Difteri": "difteri",
    "Chikungunya": "chikungunya",
    "Leptospirosis": "leptospirosis",
    "Kolera": "kolera",
    "Gizi Buruk": "gizi_buruk",
    "Jantung": "jantung",
    "TBC Paru-Paru": "tbc_paru_paru",
    "Kanker": "kanker",
    "Diabetes": "diabetes",
    "Lumpuh": "lumpuh",
    "Lainnya": "lainnya",
}

# resourceId
PENYAKIT_COL = {
    # Kesehatan - Penyakit
    "mutaber_diare": "com.kemendes.survey:id/cbMuntaber",
    "demam_berdarah": "com.kemendes.survey:id/cbDemamBerdarah",
    "campak": "com.kemendes.survey:id/cbCampak",
    "malaria": "com.kemendes.survey:id/cbMalaria",
    "flu_burung_sars": "com.kemendes.survey:id/cbFlueBurung",
    "covid19": "com.kemendes.survey:id/cbCovid19",
    "hepatitis_b": "com.kemendes.survey:id/cbHepatisis",
    "hepatitis_e": "com.kemendes.survey:id/cbLeptospirosis",
    "difteri": "com.kemendes.survey:id/cbKolera",
    "chikungunya": "com.kemendes.survey:id/cbGiziburuk",
    "leptospirosis": "com.kemendes.survey:id/cbJantung",
    "kolera": "com.kemendes.survey:id/cbTBC",
    "gizi_buruk": "com.kemendes.survey:id/cbKanker",
    "jantung": "com.kemendes.survey:id/cbDiabetes",
    "tbc_paru_paru": "com.kemendes.survey:id/cbHepatisisE",
    "kanker": "com.kemendes.survey:id/cbDifteri",
    "diabetes": "com.kemendes.survey:id/cbChikungunya",
    "lumpuh": "com.kemendes.survey:id/cbLumpuh",
    "lainnya": "com.kemendes.survey:id/cbLainnya",
    # "BJ": "com.kemendes.survey:id/cbJaminanKesehatanIndividu",
    # "BK": "com.kemendes.survey:id/cbMelahirkan",
}


def get_data_penyakit(d: Device) -> Dict[str, Any]:
    menu_to(d, "KESEHATAN")
    data: Dict[str, Any] = dict()
    # d(text="KESEHATAN").click()
    for name, resourceId in PENYAKIT_COL.items():
        value = d_get_text(d, resourceId)
        if value != "Jumlah":
            data[name] = value
    d(className="android.widget.ScrollView").fling.vert.backward()
    return data



from typing import Any, Dict
from uiautomator2 import Device, UiObject

from sdgs_tools.aplikasi_sdgs.utils import d_get_text, menu_to, swipe_box

AKSES_KESEHATAN = {
    "Rumah Sakit": "rs",
    "Rumah Sakit Bersalin": "bersalin",
    "Poliklinik": "poliklinik",
    "Puskesmas": "puskesmas",
    "Puskesmas Pembantu": "pustu",
    "Polindes": "polindes",
    "Poskesdes": "poskesdes",
    "Posyandu": "posyandu",
    "Apotik": "apotik",
    "Toko": "toko",
}

KESEHATAN_COL = {
    # "B": ("com.kemendes.survey:id/txtNIK", ""),
    "nama": ("com.kemendes.survey:id/txtNIK", "", ""),
    "jarak": ("com.kemendes.survey:id/txtNama", "Jarak : ", " km"),
    "waktu": ("com.kemendes.survey:id/txtAlamat", "Waktu Tempuh :", " jam"),
    "kemudahan": ("com.kemendes.survey:id/txtTelpon", "Kemudahan :", ""),
    # "F": ("com.kemendes.survey:id/txtStatus", ""),
}


def get_data_kesehatan(d: Device) -> Dict[str, Dict[str, Any]]:
    menu_to(d, "AKSES KESEHATAN")
    kesehatan: Dict[str, Dict[str, Any]] = dict()
    box_daftar_kesehatan: UiObject = d(
        resourceId="com.kemendes.survey:id/itemsKesehatan"
    )
    survey_boxes: UiObject = box_daftar_kesehatan.child(
        resourceId="com.kemendes.survey:id/box"
    )
    for survey_box in survey_boxes:
        data: Dict[str, Any] = dict()
        for name, (resourceId, lstrp, rstrp) in KESEHATAN_COL.items():
            current = survey_box.child(resourceId=resourceId)
            value: str = current.info.get("text")
            if isinstance(value, str):
                value = value.lstrip(lstrp)
                data[name] = value.rstrip(rstrp)
        swipe_box(d, survey_box)
        key: str = data.pop("nama")
        kesehatan[AKSES_KESEHATAN[key]] = data
    d(className="android.widget.ScrollView").fling.vert.backward()
    return kesehatan



from enum import Enum
from sdgs_tools.dashboard.utils import make_str_to_enum, make_dunder_str

MAPPING = {
    "Ada, berfungsi": "1",
    "Ada, tidak berfungsi": "2",
    "Tidak ada": "3",
}


class Jendela(Enum):
    ADA_BERFUNGSI = "1"
    ADA_TIDAK_BERFUNGSI = "2"
    TIDAK_ADA = "3"

    from_str = classmethod(make_str_to_enum(MAPPING, "1"))
    __str__ = make_dunder_str(MAPPING)



from typing import Any, Dict, List, Optional
from uiautomator2 import Device, UiObject

from sdgs_tools.aplikasi_sdgs.utils import d_get_text, menu_to

SUBTITUTE = {
    "Palawija (jagung,kacang-kacangan,ubi-ubian,dll)": "Palawija",
    "Hortikultura (buah-buahan,sayur-sayuran,tanaman hias,obat2an)": "Hortikultura",
    "Ternak  besar lainnya (kuda, kerbau, dll)": "Ternak  besar lainnya",
    "Ayam pedaging": "Ayam pedaging",
    "Telur ayam": "Telur ayam",
    "Ternak kecil lainnya (bebek, burung, dll)": "Ternak kecil lainnya",
    "Perikanan tangkap (termasuk biota lainnya)": "Perikanan tangkap",
    "Perikanan budidaya (termasuk biota lainnya)": "Perikanan budidaya",
    "Budidaya tanaman kehutanan (jati, mahoni, sengon, dll)": "Budidaya tanaman kehutanan",
    "Pemungutan hasil hutan (madu, gaharu, buah-buahan, kayu bakar, rotan, dll)": "Pemungutan hasil hutan",
    "Penangkapan satwa liar (babi, ayam hutan, kijang, dll)": "Penangkapan satwa liar",
    "Penangkaran satwa liar (arwana, buaya, dll)": "Penangkaran satwa liar",
    "Jasa pertanian (sewa traktor, penggilingan, dll)": "Jasa pertanian",
    "Uang Pensiun": "Uang Pensiunan",
    "TKI di luar negeri": "TKI di luar negeri",
    "Sumbangan (dari keluarga, dari pemerintah)": "Sumbangan",
}

# PER BOX! resourceId
PENGHASILAN_COL = {
    # "A": "NIK",
    "sumber_penghasilan": ("com.kemendes.survey:id/txtSumber", None),
    "penghasilan_jumlah": ("com.kemendes.survey:id/txtJumlah", None),
    # "D": ("com.kemendes.survey:id/txtSatuan", None),
    "penghasilan_setahun": (
        "com.kemendes.survey:id/txtPenghasilanSetahun",
        "Penghasilan setahun : ",
    ),
    "penghasilan_diekspor": ("com.kemendes.survey:id/txtDiekspor", "Ekspor : "),
    # "G": ("com.kemendes.survey:id/txtStatus", None),
}


def get_text(box: UiObject, resourceId: str, lstrip: str = None) -> Optional[str]:
    value: Optional[str] = None
    content = box.child(resourceId=resourceId)
    if content.exists:
        value = content.info.get("text")
    if value and lstrip:
        value = value.lstrip(lstrip)
    return value


def get_data_penghasilan(d: Device) -> List[Dict[str, Any]]:
    menu_to(d, "PENGHASILAN")
    # d(text="PENGHASILAN").click()
    box_daftar_penghasilan = d(resourceId="com.kemendes.survey:id/itemsPenghasilan")
    survey_box: UiObject = box_daftar_penghasilan.child(
        resourceId="com.kemendes.survey:id/box"
    )
    results: List[Dict[str, Any]] = list()
    for box_penghasilan in survey_box:
        data: Dict[str, Any] = dict()
        for name, resourceId_lstrip in PENGHASILAN_COL.items():
            data[name] = get_text(box_penghasilan, *resourceId_lstrip)
        old_pekerjaan = data["sumber_penghasilan"]
        if old_pekerjaan in SUBTITUTE:
            data["pekerjaan_utama"] = SUBTITUTE[old_pekerjaan]
        results.append(data)
    d(className="android.widget.ScrollView").fling.vert.backward()
    return results



from enum import Enum
from sdgs_tools.dashboard.utils import make_str_to_enum, make_dunder_str

MAPPING = {
    "Darat": "1",
    "Air": "2",
    "Udara": "3",
}


class JenisTransportasi(Enum):
    DARAT = "1"
    AIR = "2"
    UDARA = "3"

    from_str = classmethod(make_str_to_enum(MAPPING, "1"))
    __str__ = make_dunder_str(MAPPING)



import attr
from datetime import datetime
from time import time
from typing import Dict, Optional


@attr.dataclass
class UserType:
    id: int
    nama: str


@attr.dataclass
class User:
    id: int
    createdBy: str
    createdOn: datetime
    email: Optional[str]
    username: str
    noHp: str
    ktp: str
    nama: str
    alamat: str
    userType: UserType
    userTypeId: int
    wilayah: str
    namaDesa: Optional[str]
    namaKecamatan: Optional[str]
    namaKota: Optional[str]
    namaProvinsi: Optional[str]

    def __str__(self):
        return self.nama


@attr.dataclass
class Token:
    access_token: str
    expires_in: int
    refresh_expires_in: int
    refresh_token: str
    token_type: str
    id_token: Optional[str]
    # not-before-policy: 0
    session_state: str
    scope: str
    error: Optional[str]
    error_description: Optional[str]
    error_uri: Optional[str]
    _expires_time: int = 0

    def __attrs_post_init__(self) -> None:
        now = int(time())
        self._expires_time = now + self.expires_in

    def is_valid(self, delta: int = -60) -> bool:
        now = int(time())
        return now >= (self._expires_time + delta)

    @property
    def token_header(self) -> Dict[str, str]:
        return {"Authorization": f"{self.token_type} {self.access_token}"}


@attr.dataclass
class AuthToken:
    token: Token
    user: User
    wilayah: str



import tkinter as tk
from tkinter.filedialog import askopenfilename
from tkinter.messagebox import showerror, showinfo, showwarning
from uiautomator2 import connect

from sdgs_tools.gui.form import TextForm
from sdgs_tools.gui.ya_tidak import YaTidakForm
from sdgs_tools.aplikasi_sdgs.export_individu import export_individu as _export_individu
from sdgs_tools.utils import parse_range


class ExportIndividuWindow(tk.Toplevel):
    def __init__(self, master=None):
        super().__init__(master=master, width=400, height=400)
        self.title("Eksport Individu")
        self.label_info = tk.Label(
            self,
            text="Eksport data individu",
        )
        self.label_info.grid(row=0, column=0, columnspan=3)
        # Baris
        self.baris = TextForm(self, "Baris", 1)
        self.baris_info_button = tk.Button(self, text="?", command=self.info_baris)
        self.baris_info_button.grid(row=1, column=2)
        # RT/RW
        self.rt_rw = TextForm(self, "RT/RW", 2)
        # Individu
        self.individu = YaTidakForm(self, "Ekspor data individu", 3, True)
        # Pekerjaan
        self.pekerjaan = YaTidakForm(self, "Ekspor data pekerjaan", 4, True)
        # Penghasilan
        self.penghasilan = YaTidakForm(self, "Ekspor data penghasilan", 5, True)
        # Kesehatan
        self.kesehatan = YaTidakForm(self, "Ekspor data kesehatan", 6, True)
        # Disabilitas
        self.disabilitas = YaTidakForm(self, "Ekspor data disabilitas", 7, True)
        # Pendidikan
        self.pendidikan = YaTidakForm(self, "Ekspor data pendidikan", 8, True)
        self.info_label = tk.Label(
            self,
            text="Buat template dan isi dengan rt rw, no kk, "
            "dan nik dari data yang ingin diekspor dari aplikasi!",
        )
        self.info_label.grid(row=9, column=0, columnspan=3)
        self.start_button = tk.Button(
            self,
            text="Pilih template individu dan mulai ekspor",
            command=self.export,
        )
        self.start_button.grid(row=10, column=0)

    def export(self):
        files = [
            ("Excel 2010+", "*.xlsx"),
            ("Excel + Macro", "*.xlsm"),
        ]
        filepath = askopenfilename(filetypes=files, defaultextension=files)
        if not filepath:
            showwarning("Gagal", "Mohon pilih template yang benar")
            return
        try:
            _export_individu(
                d=connect(),
                filepath=filepath,
                rt_rw=str(self.rt_rw),
                rows=parse_range(self.baris.get()),
                start_row_penghasilan=2,
                skip_individu=not self.individu.get(),
                skip_pekerjaan=not self.pekerjaan.get(),
                skip_penghasilan=not self.penghasilan.get(),
                skip_kesehatan=not self.kesehatan.get(),
                skip_disabilitas=not self.disabilitas.get(),
                skip_pendidikan=not self.pendidikan.get(),
            )
        except FileNotFoundError:
            showwarning(
                "Gagal", "Gagal mengeksport data karena file template tidak ditemukan"
            )
        except Exception as e:
            showwarning("Error", f"Gagal mengeksport data individu karena {repr(e)}")

    @staticmethod
    def info_baris():
        showinfo(
            "Baris?",
            'Baris data dari template yang diambil datanya\nMisalnya "2-100,150-200"',
        )



import tkinter as tk
from tkinter.filedialog import askopenfilename
from tkinter.messagebox import showerror, showinfo, showwarning
from typing import Optional


from sdgs_tools.utils import parse_range
from sdgs_tools.dashboard.sdgs import Sdgs
from sdgs_tools.dashboard.import_keluarga import import_keluarga
from ..form import TextForm


class ImportKeluargaWindow(tk.Toplevel):
    def __init__(self, master=None):
        super().__init__(master=master)
        self.geometry("230x250")
        self.sdgs: Optional[Sdgs] = None
        self.user_label: Optional[tk.Label] = None
        self.title("Import Keluarga")
        self.label_info = tk.Label(
            self,
            text="Import data keluarga",
        )
        self.label_info.grid(row=0, column=0, columnspan=3)
        # Autentikasi
        self.username_form = TextForm(self, "Username", 1)
        self.password_form = TextForm(self, "Password", 2, password=True)
        self.login_button = tk.Button(
            self,
            text="Login",
            command=self.login,
        )
        self.login_button.grid(row=3, column=2)

    def show_import(self):
        # Import?
        self.baris_form = TextForm(self, "Baris", 5)
        self.baris_info_button = tk.Button(self, text="?", command=self.info_baris)
        self.baris_info_button.grid(row=5, column=2)
        self.rt_form = TextForm(self, "Rt", 6)
        self.rw_form = TextForm(self, "Rw", 7)
        self.info_label = tk.Label(
            self,
            text="Unduh template di .....",
        )
        self.info_label.grid(row=9, column=0, columnspan=3)
        self.start_button = tk.Button(
            self,
            text="Pilih template keluarga dan mulai import",
            command=self.start_import,
        )
        self.start_button.grid(row=10, column=0, columnspan=3)

    def login(self):
        username: str = self.username_form.value.get()
        password: str = self.password_form.value.get()
        try:
            self.sdgs = Sdgs(username, password)
            showinfo(
                "Sukses",
                f"Berhasil login dengan menggunakan akun {self.sdgs.token.user}",
            )
            self.user_label = tk.Label(self, text=self.sdgs.token.user)
            self.user_label.grid(row=3, column=0, columnspan=2)
            self.show_import()
        except Exception as e:
            showerror("Gagal", f"Gagal login, alasan: {e}")

    def start_import(self):
        rt = self.rt_form.value.get()
        rw = self.rw_form.value.get()
        if len(rt) != 3:
            showerror("Gagal", "Rt harus 3 digit, misal 001")
            return
        if len(rw) != 3:
            showerror("Gagal", "Rw harus 3 digit, misal 001")
            return
        files = [
            ("Excel Macro", "*.xlsm"),
        ]
        filepath = askopenfilename(filetypes=files, defaultextension=files)
        if not filepath:
            showwarning("Gagal", "Mohon pilih template yang benar")
            return
        try:
            import_keluarga(
                sdgs=self.sdgs,
                filepath=filepath,
                rows=parse_range(self.baris_form.value.get()),
                rt=rt,
                rw=rw,
            )
        except FileNotFoundError:
            showwarning(
                "Gagal",
                "Gagal mengeksport data karena file template tidak ditemukan",
            )
        except Exception as e:
            showerror("Error", f"Terjadi error : {e}")

    @staticmethod
    def info_baris():
        showinfo(
            "Baris?",
            'Baris data dari template yang diambil datanya\nMisalnya "2-100,150-200"',
        )



import tkinter as tk


class YaTidakForm:
    def __init__(self, master, label: str, row: int, default: bool):
        # Label
        self.label = tk.Label(master, text=label)
        self.label.grid(row=row, column=0)
        # Var
        self.var = tk.BooleanVar(master, value=default)
        self.ya = tk.Radiobutton(
            master,
            text="Ya",
            variable=self.var,
            value=True,
        )
        self.ya.grid(row=row, column=1)
        self.tidak = tk.Radiobutton(
            master,
            text="Tidak",
            variable=self.var,
            value=False,
        )
        self.tidak.grid(row=row, column=2)

    @property
    def value(self) -> bool:
        return self.var.get()

    def get(self) -> bool:
        return self.var.get()



from enum import Enum
from sdgs_tools.dashboard.utils import make_str_to_enum, make_dunder_str

MAPPING = {
    "Wifi": "1",
    "Handphone": "2",
    "Wifi Dan Handphone": "3",
}


class AksesInternet(Enum):
    WIFI = "1"
    HANDPHONE = "2"
    WIFI_HANDPHONE = "3"

    from_str = classmethod(make_str_to_enum(MAPPING))
    __str__ = make_dunder_str(MAPPING)



from typing import Dict

MAPPING: Dict[str, str] = {
    "provinsi": "provinsi",
    "kota": "kota",
    "kecamatan": "kecamatan",
    "desa": "desa",
    "rt": "rt",
    "rw": "rw",
    "no_kk": "no_kk",
    "nik": "nik",
    "K.P206": "nama",
    "K.P207": "alamat",
    "K.P208": "nomor_hp",
    "K.P209": "telepon_rumah",
    "K.P303": "anggota_keluarga",
    "K.P401": "tempat_tinggal",
    "K.P401-Comment": "tempat_tinggal_comment",
    "K.P402": "status_lahan",
    "K.P402-Comment": "status_lahan_comment",
    "K.P403": "luas",
    "K.P404": "lantai",
    "K.P404-Comment": "lantai_other",
    "K.P405": "dinding",
    "K.P405-Comment": "dinding_other",
    "K.P406": "jendela",
    "K.P407": "atap",
    "K.P407-Comment": "atap_comment",
    "K.P408": "penerangan",
    "K.P409": "energi_memasak",
    "K.P410": "sumber_kayu_bakar",
    "K.P411": "pembuangan_sampah",
    "K.P412": "fasilitas_mck",
    "K.P413": "sumber_air_mandi",
    "K.P413-Comment": "sumber_air_mandi_comment",
    "K.P414": "fasilitas_bab",
    "K.P414-Comment": "fasilitas_bab_comment",
    "K.P415": "sumber_air_minum",
    "K.P415-Comment": "sumber_air_minum_comment",
    "K.P416": "pembuangan_limbah_cair",
    "K.P416-Comment": "pembuangan_limbah_cair_comment",
    "K.P417": "bawah_sutet",
    "K.P418": "bantaran_sungai",
    "K.P419": "lereng_gunung",
    "K.P420": "kumuh",
    # "K.P421": "akses_pendidikan",
    # "K.P422": "akses_fasilitas_kesehatan",
    # "K.P423": "akses_tenaga_kesehatan",
    # "K.P424": "akses_sarpras_transport",
    "K.P425": "transport_umum",
    "K.P426": "transport_umum_bulan_sebelumnya",
    "K.P427": "penerima_program_pemerintah",
    "K.P428": "pengeluaran_bulanan",
}

MAPPING_COLS: Dict[str, str] = {
    "no_kk": "A",
    "nik": "B",
    "nama": "C",
    "alamat": "D",
    "nomor_hp": "E",
    "telepon_rumah": "F",
    "anggota_keluarga": "G",
    "tempat_tinggal": "H",
    "tempat_tinggal_comment": "I",
    "status_lahan": "J",
    "status_lahan_comment": "K",
    # "luas": "LM",
    "lantai": "N",
    "lantai_other": "O",
    "dinding": "P",
    "dinding_other": "Q",
    "jendela": "R",
    "atap": "S",
    "atap_comment": "T",
    "penerangan": "U",
    "energi_memasak": "V",
    "sumber_kayu_bakar": "W",
    "pembuangan_sampah": "X",
    "fasilitas_mck": "Y",
    "sumber_air_mandi": "Z",
    "sumber_air_mandi_comment": "AA",
    "fasilitas_bab": "AB",
    "fasilitas_bab_comment": "AC",
    "sumber_air_minum": "AD",
    "sumber_air_minum_comment": "AE",
    "pembuangan_limbah_cair": "AF",
    "pembuangan_limbah_cair_comment": "AG",
    "bawah_sutet": "AH",
    "bantaran_sungai": "AI",
    "lereng_gunung": "AJ",
    "kumuh": "AK",
    # "akses_pendidikan": "",
    # "akses_fasilitas_kesehatan": "",
    # "akses_tenaga_kesehatan": "",
    # "akses_sarpras_transport": "",
    "transport_umum": "EJ",
    "transport_umum_bulan_sebelumnya": "EK",
    "penerima_program_pemerintah": "EL",
    "pengeluaran_bulanan": "EM",
}



import attr
from typing import Dict, List, Optional

from .enums import YaTidak


MAPPING = {
    "Tunanetra": "tunanetra",
    "Tunarungu": "tunarungu",
    "Tunawicara": "tunawicara",
    "Tunarungu Wicara": "tunarungu_wicara",
    "Tunadaksa": "tunadaksa",
    "Tunagrahita": "tunagrahita",
    "Tunalaras": "tunalaras",
    "Cacat Eks Kusta": "cacat_eks_kusta",
    "Cacat Ganda": "cacat_ganda",
    "Dipasung": "dipasung",
}


@attr.dataclass
class Disabilitas:
    tunanetra: YaTidak = YaTidak.TIDAK
    tunarungu: YaTidak = YaTidak.TIDAK
    tunawicara: YaTidak = YaTidak.TIDAK
    tunarungu_wicara: YaTidak = YaTidak.TIDAK
    tunadaksa: YaTidak = YaTidak.TIDAK
    tunagrahita: YaTidak = YaTidak.TIDAK
    tunalaras: YaTidak = YaTidak.TIDAK
    cacat_eks_kusta: YaTidak = YaTidak.TIDAK
    cacat_ganda: YaTidak = YaTidak.TIDAK
    dipasung: YaTidak = YaTidak.TIDAK

    def todict(self) -> Dict[str, str]:
        return {
            "1": self.tunanetra.value,
            "2": self.tunarungu.value,
            "3": self.tunawicara.value,
            "4": self.tunarungu_wicara.value,
            "5": self.tunadaksa.value,
            "6": self.tunagrahita.value,
            "7": self.tunalaras.value,
            "8": self.cacat_eks_kusta.value,
            "9": self.cacat_ganda.value,
            "10": self.dipasung.value,
            "11": "2",
            "12": "2",
            "13": "2",
            "14": "2",
            "15": "2",
            "16": "2",
            "17": "2",
            "18": "2",
            "19": "2",
        }

    @classmethod
    def from_str(cls, val: Optional[str], t=None) -> "Disabilitas":
        if not val:
            return cls()
        elif isinstance(val, dict):
            return cls.from_dict(val)
        data: Dict[str, YaTidak] = dict()
        for key, name in MAPPING.items():
            if key in val:
                data[name] = YaTidak.YA
            else:
                data[name] = YaTidak.TIDAK
        return cls(**data)

    @classmethod
    def from_dict(cls, val: Dict[str, str]) -> "Disabilitas":
        data: Dict[str, YaTidak] = dict()
        for name, value in val.items():
            if value == "Ya":
                data[name] = YaTidak.YA
            else:
                data[name] = YaTidak.TIDAK
        return cls(**data)

    def __str__(self):
        values: List[str] = list()
        for key, name in MAPPING.items():
            value = getattr(self, name)
            if value is None:
                continue
            if value == YaTidak.YA:
                values.append(key)
        if values:
            return ", ".join(values)
        return ""



from enum import Enum
from sdgs_tools.dashboard.utils import make_str_to_enum, make_dunder_str

MAPPING = {
    "Semen/beton/kayu berkualitas tinggi": "1",
    "Kayu berkualitas rendah/bamboo": "2",
    "Lainnya": "other",
}


class Dinding(Enum):
    SEMEN_BETON_KAYU_KT = "1"
    KAYU_BAMBU_KR = "2"
    LAINNYA = "other"

    from_str = classmethod(make_str_to_enum(MAPPING, "other"))
    __str__ = make_dunder_str(MAPPING)



from datetime import datetime
from typing import Any, Dict
from uiautomator2 import Device, UiObjectNotFoundError

from sdgs_tools.aplikasi_sdgs.utils import d_get_text, menu_to

INDIVIDU_COL = {
    "nama": ("com.kemendes.survey:id/txtNama", None),
    "jenis_kelamin": ("com.kemendes.survey:id/cbJenisKelamin", None),
    "tempat_lahir": ("com.kemendes.survey:id/txtTempatLahir", None),
    "tanggal_lahir": ("com.kemendes.survey:id/txtTglLahir", None),
    "usia": ("com.kemendes.survey:id/txtUsia", None),
    "status_pernikahan": ("com.kemendes.survey:id/cbStatus", None),
    "usia_menikah": ("com.kemendes.survey:id/txtUsiaSaatNikah", None),
    "agama": ("com.kemendes.survey:id/cbAgama", None),
    "suku_bangsa": ("com.kemendes.survey:id/txtSuku", None),
    "warga_negara": ("com.kemendes.survey:id/cbWargaNegara", None),
    "nomor_hp": ("com.kemendes.survey:id/txtNoHP", None),
    "nomor_whatsapp": ("com.kemendes.survey:id/txtNoWA", "Nomor Untuk Whatsapp"),
    "alamat_email": ("com.kemendes.survey:id/txtEmail", "Email"),
    "alamat_facebook": ("com.kemendes.survey:id/txtFB", "Facebook"),
    "alamat_twitter": ("com.kemendes.survey:id/txtTwitter", "Twitter"),
    "alamat_instagram": ("com.kemendes.survey:id/txtInstagram", "Instagram"),
    "aktif_internet": ("com.kemendes.survey:id/cbInternet", None),
    # "akses_melalui": "com.kemendes.survey:id/cbAksesInternetLewat",
    # "kecepatan_internet": "com.kemendes.survey:id/cbKecepatanInternet",
}


def get_data_individu(d: Device) -> Dict[str, Any]:
    menu_to(d, "DATA INDIVIDU")
    data: Dict[str, Any] = dict()
    # d(text="DATA INDIVIDU").click()
    for name, (resourceId, default) in INDIVIDU_COL.items():
        try:
            value = d_get_text(d, resourceId)
            if default and value == default:
                continue
            data[name] = d_get_text(d, resourceId)
        except UiObjectNotFoundError:
            continue
    if data["aktif_internet"] == "Ya":
        # Internet = Ya
        data["akses_melalui"] = d_get_text(
            d, "com.kemendes.survey:id/cbAksesInternetLewat"
        )
        data["kecepatan_internet"] = d_get_text(
            d, "com.kemendes.survey:id/cbKecepatanInternet"
        )
    tanggal, bulan, tahun = data["tanggal_lahir"].split("/")
    data["tanggal_lahir"] = datetime(int(tahun), int(bulan), int(tanggal)).date()
    d(className="android.widget.ScrollView").fling.vert.backward()
    return data



from typing import Any, Dict
from uiautomator2 import Device

from sdgs_tools.aplikasi_sdgs.utils import d_get_text, menu_to

SUBTITUTE = {
    "Nelayan Pemilik Kapal/Perahu": "Nelayan Pemilik Kapal",
    "Nelayan Penyewa Kapal/Perahu": "Nelayan Penyewa Kapal",
    "Pensiunan": "Lainnya",
}

# EXCEL COL : d(resourceId="{}")
PEKERJAAN_COL = {
    "kondisi_pekerjaan": "com.kemendes.survey:id/cbKondisiPekerjaan",
    "pekerjaan_utama": "com.kemendes.survey:id/cbPekerjaanUtama",
    "jamsos_ketenagakerjaan": "com.kemendes.survey:id/cbJaminanSosial",
    "pekerjaan_penghasilan": "com.kemendes.survey:id/txtPenghasilan",
}


def get_data_pekerjaan(d: Device) -> Dict[str, Any]:
    menu_to(d, "PEKERJAAN")
    data: Dict[str, Any] = dict()
    # d(text="PEKERJAAN").click()
    for name, resourceId in PEKERJAAN_COL.items():
        data[name] = d_get_text(d, resourceId)
    d(className="android.widget.ScrollView").fling.vert.backward()
    old_pekerjaan = data["pekerjaan_utama"]
    if old_pekerjaan in SUBTITUTE:
        data["pekerjaan_utama"] = SUBTITUTE[old_pekerjaan]
    if data["jamsos_ketenagakerjaan"] == "Peserta":
        data["jamsos_ketenagakerjaan"] = "Ya"
    else:
        data["jamsos_ketenagakerjaan"] = "Tidak"
    return data



import click

from .template import template_individu
from .template import template_keluarga

from .individu import import_individu
from .keluarga import import_keluarga


@click.group("dashboard")
def dashboard():
    pass


dashboard.add_command(template_individu)
dashboard.add_command(template_keluarga)

dashboard.add_command(import_individu)
dashboard.add_command(import_keluarga)



import tkinter as tk


class ContentFrame(tk.Frame):
    def __init__(self, master, **kw):
        super().__init__(master, bg="#CCC", width=500, height=500, **kw)
        self.pack(expand=True, fill="both", side="right")



from typing import Any, Dict
from uiautomator2 import Device, UiObject

from sdgs_tools.aplikasi_sdgs.utils import d_get_text, menu_to, swipe_box

AKSES_PENDIDIKAN = {
    "PAUD": "paud",
    "TK/RA": "tk",
    "SD/MI atau sederajat": "sd",
    "SMP/MTs atau sederajat": "smp",
    "SMA/MA atau sederajat": "sma",
    "Perguruan Tinggi": "pt",
    "Pesantren": "pesantren",
    "Seminari": "seminari",
    "Pendidikan Keagamaan Lain": "lainnya",
}

PENDIDIKAN_COL = {
    # "B": ("com.kemendes.survey:id/txtNIK", ""),
    "nama": ("com.kemendes.survey:id/txtNIK", "", ""),
    "jarak": ("com.kemendes.survey:id/txtNama", "Jarak : ", " km"),
    "waktu": ("com.kemendes.survey:id/txtAlamat", "Waktu Tempuh :", " jam"),
    "kemudahan": ("com.kemendes.survey:id/txtTelpon", "Kemudahan :", ""),
    # "F": ("com.kemendes.survey:id/txtStatus", ""),
}


def get_data_pendidikan(d: Device) -> Dict[str, Dict[str, Any]]:
    menu_to(d, "AKSES PENDIDIKAN")
    pendidikan: Dict[str, Dict[str, Any]] = dict()
    box_daftar_pendidikan: UiObject = d(
        resourceId="com.kemendes.survey:id/itemsPendidikan"
    )
    survey_boxes: UiObject = box_daftar_pendidikan.child(
        resourceId="com.kemendes.survey:id/box"
    )
    for survey_box in survey_boxes:
        data: Dict[str, Any] = dict()
        for name, (resourceId, lstrp, rstrp) in PENDIDIKAN_COL.items():
            current = survey_box.child(resourceId=resourceId)
            # if not current.exists:
            #     continue
            value: str = current.info.get("text")
            if isinstance(value, str):
                value = value.lstrip(lstrp)
                data[name] = value.rstrip(rstrp)
        swipe_box(d, survey_box)
        pend: str = data.pop("nama")
        pendidikan[AKSES_PENDIDIKAN[pend]] = data
    d(className="android.widget.ScrollView").fling.vert.backward()
    return pendidikan



from enum import Enum
from sdgs_tools.dashboard.utils import make_str_to_enum, make_dunder_str

MAPPING = {
    "Marmer/granit": "1",
    "Keramik": "2",
    "Parket/vinil/permadani": "3",
    "Ubin/tegel/teraso": "4",
    "Kayu/papan kualitas tinggi": "5",
    "Semen/bata merah": "6",
    "Bambu": "7",
    "Kayu/papan kualitas rendah": "8",
    "Lainnya": "other",
}


class Lantai(Enum):
    MARMER_GRANIT = "1"
    KERAMIK = "2"
    PARKET_VINIL_PERMADANI = "3"
    UBIN_TEGEL_TERASO = "4"
    KAYU_PAPAN_KUALITAS_TINGGI = "5"
    SEMEN_BATA_MERAH = "6"
    BAMBU = "7"
    KAYU_PAPAN_KUALITAS_RENDAH = "8"
    LAINNYA = "other"

    from_str = classmethod(make_str_to_enum(MAPPING, "other"))
    __str__ = make_dunder_str(MAPPING)



import click

from .export import export_individu
from .export import export_keluarga


@click.group("aplikasi")
def aplikasi():
    pass


aplikasi.add_command(export_individu, "export-individu")
aplikasi.add_command(export_keluarga, "export-keluarga")



from enum import Enum
from sdgs_tools.dashboard.utils import make_str_to_enum, make_dunder_str

MAPPING = {
    "Pembelian": "1",
    "Diambil dari hutan": "2",
    "Diambil di luar/bukan hutan": "3",
    "Lainnya": "other",
}


class SumberKayuBakar(Enum):
    BELI = "1"
    HUTAN = "2"
    BUKAN_HUTAN = "3"
    LAINNYA = "other"

    from_str = classmethod(make_str_to_enum(MAPPING, "1"))
    __str__ = make_dunder_str(MAPPING)



import attr
from openpyxl.worksheet.worksheet import Worksheet
from typing import Dict


MAPPING_COLS = {
    # Fasilitas Kesehatan
    "rumah_sakit": "AC",
    "rumah_sakit_bersalin": "AD",
    "puskesmas_rawat_inap": "AE",
    "puskesmas_tanpa_inap": "AF",
    "puskesmas_pembantu": "AG",
    "poliklinik": "AH",
    "tempat_praktik_dokter": "AI",
    "rumah_bersalin": "AJ",
    "tempat_praktik_bidan": "AK",
    "poskesdes": "AL",
    "polindes": "AM",
    "apotik": "AN",
    "toko_obat_jamu": "AO",
    "posyandu": "AP",
    "posbindu": "AQ",
    "tempat_praktik_dukun": "AR",
}


@attr.dataclass
class FasilitasKesehatan:
    rumah_sakit: int = 0
    rumah_sakit_bersalin: int = 0
    puskesmas_rawat_inap: int = 0
    puskesmas_tanpa_inap: int = 0
    puskesmas_pembantu: int = 0
    poliklinik: int = 0
    tempat_praktik_dokter: int = 0
    rumah_bersalin: int = 0
    tempat_praktik_bidan: int = 0
    poskesdes: int = 0
    polindes: int = 0
    apotik: int = 0
    toko_obat_jamu: int = 0
    posyandu: int = 0
    posbindu: int = 0
    tempat_praktik_dukun: int = 0

    def todict(self) -> Dict[str, str]:
        return {
            "1": str(self.rumah_sakit),
            "2": str(self.rumah_sakit_bersalin),
            "3": str(self.puskesmas_rawat_inap),
            "4": str(self.puskesmas_tanpa_inap),
            "5": str(self.puskesmas_pembantu),
            "6": str(self.poliklinik),
            "7": str(self.tempat_praktik_dokter),
            "8": str(self.rumah_bersalin),
            "9": str(self.tempat_praktik_bidan),
            "10": str(self.poskesdes),
            "11": str(self.polindes),
            "12": str(self.apotik),
            "13": str(self.toko_obat_jamu),
            "14": str(self.posyandu),
            "15": str(self.posbindu),
            "16": str(self.tempat_praktik_dukun),
        }

    @staticmethod
    def make(ws: Worksheet, row: int):
        data: Dict[str, str] = dict()
        for nama, col in MAPPING_COLS.items():
            data[nama] = ws[f"{col}{row}"].value
        return data

    def save(self, ws: Worksheet, row):
        for nama, col in MAPPING_COLS.items():
            ws[f"{col}{row}"] = getattr(self, nama)
